/****************************************************************************
**  Copyright (C) 2004                                                     **
**  University of Tennessee, Innovative Computing Laboratory               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

/*******************************************
     Callsite.cpp

********************************************/

#include "Callsite.h"
#include "Module.h"
#include "Cnode.h"
#include "CUBException.h"
#include "Region.h"

using namespace std;
using namespace cube;


/* get the ith cnode pointing to this callsite */
const Cnode*  Callsite::get_cnode(int i) const {
  if (i < 0 || i >= num_cnodes()) 
    throw CUBException("csite vector out of range");
  return cnodev[i]; 
}

void Callsite::writeXML(ostream& out, int& id, int mod, int callee) const {
  int level;
  level = get_level();
  out << indent() << "<csite id=\"" << id++ << "\">\n";
  set_level(level+1);
  out << indent() << "<line>" << get_line() << "</line>\n";
  /*callee = (Region*)csite.get_callee();
  if (callee == NULL) {
    cout << "Error in callsite id: " << csite.get_id() << endl;
    callee = (Region*) csite.get_caller();
    //exit(1);
    }*/
  out << indent() << "<mod>"  << mod << "</mod>\n";
  out << indent() << "<callee>"  << callee << "</callee>\n";

  set_level(level);
  out << indent() << "</csite>\n";

}

namespace cube {
  bool operator== (const Callsite& a, const Callsite& b) {
    return 
      *a.get_mod() == *b.get_mod() && 
      *a.get_callee() == *b.get_callee() &&
      (a.get_line() - b.get_line()) >= -5 && // OK if line diff <=5
      (a.get_line() - b.get_line()) <= 5;
        
  }

}
