/****************************************************************************
**  Copyright (C) 2004                                                     **
**  University of Tennessee, Innovative Computing Laboratory               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#ifndef CUBE_CALLSITE_H
#define CUBE_CALLSITE_H

/*
 *----------------------------------------------------------------------------
 *
 * class Callsite
 *
 *----------------------------------------------------------------------------
 */


#include <iostream>
#include <vector>

#include "Vertex.h"

namespace cube {

  class Module;
  class Region;
  class Cnode;
  
  class Callsite: public Vertex {

  public:

    Callsite(Module* mod, int line, Region* callee) : 
      mod(mod), line(line), callee(callee) {}
    
    int get_line()              const { return line; }
    const Module* get_mod()     const { return mod; };
    const Region* get_caller()  const { return caller; }
    const Region* get_callee()  const { return callee; }

    int num_children()           const { return 0; }
    const Vertex* get_child(int) const { return NULL; }
    // a callsite may point to several cnodes.
    void add_cnode(Cnode* cnode) { cnodev.push_back(cnode); }
    const Cnode*  get_cnode(int)  const;
    int   num_cnodes()            const { return cnodev.size(); }
    

    // void set_callee(Region* r)   { callee = r; }
    void set_caller(Region* r)   { caller = r; }
    void set_mod(Module* m)      { mod = m; }

    void writeXML(std::ostream& out, int& id, int mod_id, int callee_id) const;
    
 private:
  
  int line;
  Region* caller;
  Module* mod;
  Region* callee;
  std::vector<Cnode*> cnodev;

  };
  
  bool operator== (const Callsite& a, const Callsite& b);
  
}

#endif
