/****************************************************************************
**  Copyright (C) 2004                                                     **
**  University of Tennessee, Innovative Computing Laboratory               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

/*******************************************
     Cnode.cpp

********************************************/

#include "Cnode.h"
#include "CUBException.h"
#include "Callsite.h"
#include "Region.h"

using namespace std;
using namespace cube;

Cnode::Cnode(Callsite* csite, Cnode* parent) : csite(csite), parent(parent) {
  if (get_csite()->get_callee() == NULL) {
    throw CUBException("Error: cnode has a csite pointing to a NULL callee.");
  }
  // compute the call-path strings in the reverse order.
  rpathv.push_back(get_csite()->get_callee()->get_name());
  Cnode* p = parent;
  while(p != NULL) {
    rpathv.push_back(p->get_csite()->get_callee()->get_name());
    p = const_cast<Cnode*>(p->get_parent());
  }
}

const Cnode* Cnode::get_child(int i) const {
  if (i < 0 || i >= childv.size()) 
    throw CUBException("cnode vector out of range");
  return childv[i]; 
}

void Cnode::writeXML(ostream& out, int& id, map<const Cnode*, int>& cnode_mapping,
		     map<const Callsite*, int>& csites) const {
  int level;
  int csite_id;

  level = get_level();
  csite_id = csites[csite];
  cnode_mapping[this] = id;
  out << indent() << "<cnode id=\"" << id++ << "\" csiteId=\"" << csite_id << "\">\n";
  
  for(int i = 0; i < num_children() ; i++) {
    const Cnode* cn = get_child(i);
    cn->set_level(level+1);
    cn->writeXML(out, id, cnode_mapping, csites);
  }

  set_level(level);
  out << indent() << "</cnode>\n";
}

namespace cube {
  bool operator== (const Cnode& a, const Cnode& b) {
    return *a.get_csite() == *b.get_csite();
  }

}
