/****************************************************************************
**  Copyright (C) 2004                                                     **
**  University of Tennessee, Innovative Computing Laboratory               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#ifndef CUBE_CNODE_H
#define CUBE_CNODE_H   

/*
 *----------------------------------------------------------------------------
 *
 * class Cnode
 *
 *----------------------------------------------------------------------------
 */

#include <iostream>
#include <vector>
#include <map>
#include <functional>

#include "Vertex.h"


namespace cube {
  class Callsite;
  
  class Cnode: public Vertex {
    
  public:
    
    Cnode(Callsite* csite, Cnode* parent);

    void add_child(Cnode* child)      { childv.push_back(child); }
    int num_children()  const         { return childv.size(); }
    const Cnode* get_child(int i)     const;
    
    const std::vector<Cnode*>& get_children() const { return childv; }
    const Cnode*    get_parent()              const { return parent; }
    const Callsite* get_csite()               const { return csite;  }
    std::vector<std::string> get_rcpath()     const { return rpathv; }

    // csites is an input argument, cnode_mapping an output.
    void writeXML(std::ostream& out, int& id, 
		  std::map<const Cnode*, int>& cnode_mapping, 
		  std::map<const Callsite*, int>& csites) const;


  private:
    Callsite* csite;   
    std::vector<Cnode*> childv;
    Cnode* parent;
    std::vector<std::string> rpathv; // reverse call-path from itself to root

  };
  
  bool operator== (const Cnode& a, const Cnode& b);

  //binary predicate
  class Less : public std::binary_function<Cnode*, Cnode*, bool> {
    public :
      bool operator() (const Cnode* a, const Cnode* b) const {
      return (a->get_rcpath().size() <= b->get_rcpath().size());
    }
  };
  
}

#endif
