/****************************************************************************
**  Copyright (C) 2004                                                     **
**  University of Tennessee, Innovative Computing Laboratory               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#ifndef CUBE_CUBECOLLECT_H
#define CUBE_CUBECOLLECT_H   

/*
 *----------------------------------------------------------------------------
 *
 * class CubeCollect
 *
 *----------------------------------------------------------------------------
 */

#include <map>
#include <vector>

#include "Matrix.h"

#define CUBE_INCL    0
#define CUBE_EXCL    1

#define CUBE_MET     0
#define CUBE_PROG    1
#define CUBE_LOC     2

#define TOL 1.0e-12

namespace cube {
  class Cube;
  class Metric;
  class Cnode;
  class Location;
  class Module;
  class Region;
  class Thread;

  class CubeCollect{
    
  public:
    CubeCollect(Cube* cube);
    ~CubeCollect();
    
    Cube* get_cube() { return cube; }

    // call-tree node severity
    double get_vcsev   (int mode[3], const Metric* met, 
			const Cnode* cnode = NULL, 
			const Location* loc = NULL);
    
    // region severity
    double get_vrsev   (int mode[3], const Metric* met, const Region* reg, 
			const Location* loc = NULL);
    
    // module severity
    double get_vmsev   (int mode[3], const Metric* met, const Module* mod, 
			const Location* loc = NULL);
    
    
  private:

    Cube* cube;
    bool excl_met_flag;
    
    // extended severity
    double get_esev    (const Metric* met, const Cnode* cnode, const Location* loc);
    // metric with exclusive mode
    double get_esev_ex (const Metric* met, const Cnode* cnode, const Location* loc);

    double sev(int x, int y, int z, const Metric* b, const Cnode* n, 
	       const Location* l);
    double sev(int y, int z, const Metric* b, const Cnode* n, 
	       const Location* l);
    double sev(int z, const Metric* b, const Cnode* n, const Location* l);
    
    // return NULL if not a thread.
    const Thread*        loc2thrd(const Location* loc);    
    // called once for each root metric
    Matrix               calc_excl_matrix(const Metric* root);
    // for flat tree
    std::vector<const Cnode*>  get_callees(const Region* regn);
    
    // exclusive matrices for performance metrics.
    std::map<const Metric*, Matrix > excl_severity;    
  
  };
  
}

#endif
