/****************************************************************************
**  Copyright (C) 2004                                                     **
**  University of Tennessee, Innovative Computing Laboratory               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

/*******************************
				
     CnodeTree.cpp	
					
********************************/

#include <iostream>
#include <fstream>

#include "Tree.h"
#include "MidTree.h"
#include "CnodeTree.h"
#include "LocationTree.h"

#include "PresenterApp.h"
#include "PresenterFrame.h"
#include "SourceCodeDlg.h"
#include "GUIData.h"
#include "CubeCollect.h"

#include "Cnode.h"
#include "Callsite.h"
#include "Region.h"
#include "Module.h"
#include "Browser.h"

using namespace std;
using namespace cube;

BEGIN_EVENT_TABLE(CnodeTree, MidTree)
  EVT_MENU       (CALLTREE_POP_ID_Loc,   CnodeTree::OnCalleeLoc)
  EVT_MENU       (CALLTREE_POP_ID_Code,  CnodeTree::OnCalleeSourceCode)
END_EVENT_TABLE()

CnodeTree::CnodeTree(wxWindow* parent) : MidTree(parent) { 
  popup_menu->Delete(MID_POP_ID_Des);
  popup_menu->Delete(MID_POP_ID_URL);
  /* not done yet!!!
  popup_menu->Delete(MID_POP_ID_Code);
  popup_menu->Delete(MID_POP_ID_Loc);

  wxMenu* sub_menu1 = new wxMenu;
  wxMenu* sub_menu2 = new wxMenu;
  popup_menu->Append(101, "Show call site", sub_menu1);
  popup_menu->AppendSeparator();
  popup_menu->Append(102, "Show callee", sub_menu2);
  
  // for call site
  sub_menu1->Append(MID_POP_ID_Loc,   _T("Location"));
  sub_menu1->Append(MID_POP_ID_Code,  _T("Source code"));  

  // for region
  sub_menu2->Append(CALLTREE_POP_ID_Loc,   _T("Location"));
  sub_menu2->Append(CALLTREE_POP_ID_Code,  _T("Source code")); 
  sub_menu2->Append(MID_POP_ID_URL,   _T("Online Description"));
  sub_menu2->Append(MID_POP_ID_Des,   _T("Info"));
  */
}

void CnodeTree::show_descr_dlg(const Cnode* cnode) {
  PresenterFrame* frame = myApp->get_frame();
  wxString mod_text;
  wxString line_text;
  
  const Callsite* csite = cnode->get_csite();
  wxDialog* dlg = new wxDialog(this, -1, "Call site location", 
			       wxPoint(0,0), wxSize(300, 100));
  wxBoxSizer* sizer = new wxBoxSizer(wxVERTICAL);
  const Module* m = csite->get_mod();
  
  string fname = m->get_name();
  int ln = csite->get_line();
  line_text.Printf("Line number: %d", ln);
  
  if (ln < 0) { // if line number N/A, correct it!
    const Region* caller = csite->get_caller();
    if (caller != NULL) { // borrow its caller's information
      fname = caller->get_mod()->get_name();
      int bln = caller->get_begn_ln();
      int eln = caller->get_end_ln();
      line_text = region_ln_label(bln, eln).c_str();
    }
  }
  if (fname.empty()) fname = "Undefined";
  mod_text.Printf("Module name: %s", fname.c_str());

  sizer->Add(new wxStaticText(dlg, -1, mod_text, wxDefaultPosition, 
			      wxSize(300, -1)),
	     1, wxALL, 3);
  sizer->Add(new wxStaticText(dlg, -1, line_text, wxDefaultPosition, 
			      wxSize(300, -1)),
	     1,wxALL, 3);
  // "OK" button
  sizer->Add(new wxButton(dlg, wxID_OK, _("&Ok")), 1, 
	     wxALIGN_CENTRE_HORIZONTAL, 3);
  
  dlg->SetSizer(sizer);
  dlg->Centre(wxBOTH);
  dlg->ShowModal();
  delete dlg;
}

// callee "info"
void CnodeTree::OnDes(wxCommandEvent& event) {
  PresenterFrame* frame = myApp->get_frame();
  wxString text;
  Vertex* obj = GetItemData(right_click_item)->get_obj();
  if (obj != NULL) {
    Cnode* cn = static_cast<Cnode*>(obj);
    string descr = cn->get_csite()->get_callee()->get_descr();
    text.Printf("%s\n", descr.c_str());
    wxMessageBox(text, _T("Callee Information"), wxOK, this);
  }

}
// callee url
void CnodeTree::OnURL (wxCommandEvent& event) {
  PresenterFrame* frame = myApp->get_frame();
  Vertex* obj = GetItemData(right_click_item)->get_obj();
  if (obj != NULL) {
    Cnode* cn = static_cast<Cnode*>(obj);
    string url = cn->get_csite()->get_callee()->get_url();
    if (frame->get_browser_dlg() == NULL)
      frame->set_browser_dlg(new Browser(frame));
    Browser* browser = frame->get_browser_dlg();    
    int ret = frame->load_working_url(browser, url);
    if (ret < 0) {
      cerr << "Error: " << url << " not accessible." << endl;
      return;
    }
    if (ret > 0) {
      browser->Show(true);
      browser->Raise();
    }
  }
}

// callee location
void CnodeTree::OnCalleeLoc         (wxCommandEvent& event) {
  Vertex* obj = GetItemData(right_click_item)->get_obj();
  if (obj != NULL) {
    Cnode* cn = static_cast<Cnode*>(obj);
    show_descr_dlg4reg(cn->get_csite()->get_callee());
  }

}

// callee code
void CnodeTree::OnCalleeSourceCode  (wxCommandEvent& event) {
  PresenterFrame* frame = myApp->get_frame();
  Vertex* obj = GetItemData(right_click_item)->get_obj();

  if (obj != NULL) {
    Cnode* cn = static_cast<Cnode*>(obj);
    const Region* r = cn->get_csite()->get_callee();
    if (r == NULL) return;
    const Module* m = r->get_mod();
    string mname = m->get_name();
    SourceCodeDlg* dlg = frame->get_source_dlg();
    if (dlg == NULL) {
      dlg = new SourceCodeDlg(this);
      dlg->Move(frame->GetPosition()-dlg->GetPosition()-wxPoint(400,0));
      frame->set_source_dlg(dlg);
    }
    frame->get_source_dlg()->set_module(mname);
    frame->get_source_dlg()->set_line(r->get_begn_ln());
    string ln = get_line(get_path(mname), get_base(mname), 1);
    if (ln.empty()) return; // user clicked "Cancel"
    display_region_code(r->get_begn_ln(), r->get_end_ln());  
    return;
  }

}

void CnodeTree::OnLoc(wxCommandEvent& event) {
  Vertex* obj = GetItemData(right_click_item)->get_obj();
  if (obj != NULL) {
    show_descr_dlg(static_cast<Cnode*>(obj)); // description of call-site.
  }
  else {
    cerr << "Warning: cnode right-clicked has a NULL obj" << endl;
  }
}

/******************************************************************
   Display source code accordingly 
   Policy:
   - path name defined in *.cube.
   - user-selected directory and subdirectory (if there is one)
   - pop up directory-selection dialog and let user pick directory
     (remember): current directory by default
   - the line of interest should be marked
       - if you have a line#: mark line
       - else mark its caller region

*******************************************************************/
void CnodeTree::OnSourceCode(wxCommandEvent& event) {
  PresenterFrame* frame = myApp->get_frame();
  GUIData* guidata = frame->get_guidata();
  int line = 0;       // the callsite line number
  string mname;
  Vertex* obj = GetItemData(right_click_item)->get_obj();
  if (obj == NULL) return;
  
  const Cnode* cnode = static_cast<Cnode*>(obj);
  const Callsite* csite = cnode->get_csite();
  line = csite->get_line();
  const Module* mp = csite->get_mod();
  mname = mp->get_name();
  
  SourceCodeDlg* dlg = frame->get_source_dlg();
  if (dlg == NULL) {
    dlg = new SourceCodeDlg(this);
    dlg->Move(frame->GetPosition()-dlg->GetPosition()-wxPoint(400,0));
    frame->set_source_dlg(dlg);
  }
  frame->get_source_dlg()->set_module(mname);
  frame->get_source_dlg()->set_line(line);
  if (line <= 0) {
    // user didn't define the line number, show the caller region
    const Region* caller = csite->get_caller();
    if (caller != NULL) {
      mname = caller->get_mod()->get_name();
      string ln = get_line(get_path(mname), get_base(mname), caller->get_begn_ln());
      if (ln.empty()) return; // user clicked "Cancel"
      display_region_code(caller->get_begn_ln(), caller->get_end_ln());
    }
    return;
  }

  // If a line number defined, we always assume it is correct.
  string code = get_line(get_path(mname), get_base(mname), 
			 line); // could be code, "empty line", or "N/A"
  if (code.empty()) return;
  display_region_code(line, line); // one line is highlighted
  
}

void CnodeTree::OnExpanded(wxTreeEvent& event) {
  wxTreeItemId id = event.GetItem();
  PresenterFrame* frame = myApp->get_frame();
  Vertex* obj = GetItemData(id)->get_obj();
  if (obj == NULL) {
    cerr << "Warning: cnode expanded has a NULL obj" << endl;  
    return;
  }
  const Cnode* cnode = static_cast<Cnode*>(obj);
  GUIData* guidata = frame->get_guidata();
  if (IsSelected(id))guidata->xpnd_sel_middle();
  string label = guidata->get_cnode_label(CUBE_EXCL, cnode);
  SetItemText(id, label, guidata->get_cnode_color(CUBE_EXCL, cnode));
  if (IsSelected(id)) frame->get_loc_tree()->refresh();
  
}

void CnodeTree::OnCollapsed(wxTreeEvent& event) {
  wxTreeItemId id = event.GetItem();
  PresenterFrame* frame = myApp->get_frame();
  Vertex* obj = GetItemData(id)->get_obj();
  if (obj == NULL) {
    cerr << "Warning: cnode collapsed has a NULL obj" << endl;  
    return;
  }
  const Cnode* cnode = static_cast<Cnode*>(obj);
  GUIData* guidata = frame->get_guidata();
  if (IsSelected(id))guidata->clps_sel_middle();
  string label = guidata->get_cnode_label(CUBE_INCL, cnode);
  SetItemText(id, label, guidata->get_cnode_color(CUBE_INCL, cnode));
  if (IsSelected(id)) frame->get_loc_tree()->refresh();
    
}


void CnodeTree::OnSelChanged(wxTreeEvent& event) {
  wxTreeItemId id = GetSelection(); // newly selected node.
  PresenterFrame* frame = myApp->get_frame();
  Vertex* obj = GetItemData(id)->get_obj();
  if (obj == NULL) {
    cerr << "Warning: cnode selected has a NULL obj" << endl;  
    return;
  }
  const Cnode* cnode = static_cast<Cnode*>(obj);
  GUIData* guidata = frame->get_guidata();
  guidata->select_cnode(cnode);  
  IsExpanded(id) ? guidata->xpnd_sel_middle() : guidata->clps_sel_middle();
  if (frame->get_loc_tree() != NULL)
    frame->get_loc_tree()->refresh();
}

// local function.
bool enable_src_dlg(Cnode* cn) {
  string fname;
  const Callsite* cs = cn->get_csite();
  if (cs->get_line() > 0) return true;
  const Region* caller = cs->get_caller();
  if (caller != NULL) { // borrow its caller's information
    fname = caller->get_mod()->get_name();
    int bln = caller->get_begn_ln();
    int eln = caller->get_end_ln();
    if (bln > 0 || eln > 0) return true;
    else return false;
  }
  return false;
}
void CnodeTree::OnRightClick(wxTreeEvent& event) {
  right_click_item = HitTest(event.GetPoint());
  PresenterFrame* frame = myApp->get_frame();
  Vertex* obj = GetItemData(right_click_item)->get_obj();
  if (obj == NULL) return;

  Cnode* cn = static_cast<Cnode*>(obj);
  bool  enable_src = enable_src_dlg(cn); // caller source

  const Region* r  = cn->get_csite()->get_callee();
  string url       = r->get_url();
  string des       = r->get_descr();
  bool  no_callee_src = r->get_begn_ln() < 0 && r->get_end_ln();

  // callsite may have no "source code"
  if (!enable_src) popup_menu->Enable(MID_POP_ID_Code, false);
  // callee's menu items:
  if (no_callee_src) 
    popup_menu->Enable(CALLTREE_POP_ID_Code, false); // callee source code
  if (url.empty()) popup_menu->Enable(MID_POP_ID_URL, false); // callee url
  if (des.empty()) popup_menu->Enable(MID_POP_ID_Des, false); // callee info
  PopupMenu(popup_menu, event.GetPoint());
  if (url.empty()) popup_menu->Enable(MID_POP_ID_URL, true);
  if (!enable_src) popup_menu->Enable(MID_POP_ID_Code,true);
  if (des.empty()) popup_menu->Enable(MID_POP_ID_Des, true);
  if (no_callee_src) popup_menu->Enable(CALLTREE_POP_ID_Code, true);
  
  /*
  Cnode* cn = static_cast<Cnode*>(obj);
  string url = cn->get_csite()->get_callee()->get_url();
  if(!url.empty()) {//dynamically add menu item
    string text = "Goto URL";
    popup_menu->Append(MID_POP_ID_URL, text.c_str());
  }
  PopupMenu(popup_menu, event.GetPoint());
  if(!url.empty()) {//dynamically remove menu item
    popup_menu->Delete(MID_POP_ID_URL);
  }
  */
}

void CnodeTree::refresh () {
  if (GetChildrenCount(GetVRootItem(), false) == 0) return;
  traverse(GetVRootItem());
#ifdef CUBE_DEBUG
  cout << "call tree is refreshed." << endl;
#endif
  PresenterFrame* frame = myApp->get_frame();
  if (frame->get_loc_tree() != NULL)
    frame->get_loc_tree()->refresh();
}

void CnodeTree::traverse(wxTreeItemId root) {
  long cookie; // wxWindows's variable
  PresenterFrame* frame = myApp->get_frame();
  GUIData* guidata = frame->get_guidata();
  Vertex* obj = GetItemData(root)->get_obj();
  
  if (obj != NULL) {
    const Cnode* cnode = static_cast<Cnode*>(obj);
    int mode = IsExpanded(root) ? CUBE_EXCL : CUBE_INCL;
    string label = guidata->get_cnode_label(mode, cnode);
    SetItemText(root, label, guidata->get_cnode_color(mode, cnode));
  }
  if (GetChildrenCount(root) == 0) {
    return;
  }
  wxTreeItemId child = GetFirstChild(root, cookie); 
  traverse(child);
  while (child = GetNextChild(root, cookie)) {
    traverse(child);
  }
  return;
}


