/****************************************************************************
**  Copyright (C) 2004                                                     **
**  University of Tennessee, Innovative Computing Laboratory               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

/*******************************
				
    LocationTree.cpp	
					
********************************/

#include <iostream>

#include "Tree.h"
#include "LocationTree.h"
#include "PresenterApp.h"
#include "PresenterFrame.h"
#include "Location.h"

#include "GUIData.h"
#include "CubeCollect.h"

using namespace std;
using namespace cube;

LocationTree::LocationTree(wxWindow* parent) : Tree(parent) {
  popup_menu->Append(Loc_POP_ID_Item1, _T("No Menu Item"));
}

void LocationTree::OnExpanded(wxTreeEvent& event) {
  wxTreeItemId id = event.GetItem();
  PresenterFrame* frame = myApp->get_frame();
  Vertex* obj = GetItemData(id)->get_obj();
  if (obj == NULL) {
    cerr << "Warning: loc expanded has a NULL obj" << endl;  
    return;
  }
  GUIData* guidata = frame->get_guidata();
  const Location* loc = static_cast<Location*>(obj);
  string label = guidata->get_loc_label(CUBE_EXCL, loc);
  SetItemText(id, label, guidata->get_loc_color(CUBE_EXCL, loc));
}

void LocationTree::OnCollapsed(wxTreeEvent& event) {
  wxTreeItemId id = event.GetItem();
  PresenterFrame* frame = myApp->get_frame();
  Vertex* obj =  GetItemData(id)->get_obj();
  if (obj == NULL) {
    cerr << "Warning: loc collapsed has a NULL obj" << endl;  
    return;
  }
  GUIData* guidata = frame->get_guidata();
  const Location* loc = static_cast<Location*>(obj);
  string label = guidata->get_loc_label(CUBE_INCL, loc);
  SetItemText(id, label, guidata->get_loc_color(CUBE_INCL, loc));
}

void LocationTree::OnSelChanged  (wxTreeEvent& event) {
  SelectItem(GetVRootItem());
} 

void LocationTree::refresh() {
  if (GetChildrenCount(GetVRootItem(), false) == 0) return;
  traverse(GetVRootItem());
#ifdef CUBE_DEBUG
  cout << "loc-tree is refreshed:"<< endl;
#endif
  PresenterFrame* frame = myApp->get_frame();
  GUIData* guidata = frame->get_guidata();
#ifdef CUBE_DEBUG
  guidata->print_info();
#endif
}

void LocationTree::traverse(wxTreeItemId root) {
  long cookie;
  PresenterFrame* frame = myApp->get_frame();
  GUIData* guidata = frame->get_guidata();
  Vertex* obj = GetItemData(root)->get_obj();
  
  if(obj != NULL) {
    const Location* loc = static_cast<Location*>(obj);
    int mode = IsExpanded(root) ? CUBE_EXCL : CUBE_INCL;
    string label = guidata->get_loc_label(mode, loc);
    SetItemText(root, label, guidata->get_loc_color(mode, loc));
  }

  if (GetChildrenCount(root) == 0) {
    return;
  }
  wxTreeItemId child = GetFirstChild(root, cookie); 
  traverse(child);
  while (child = GetNextChild(root, cookie)) {
    traverse(child);
  }
  return;
}


void LocationTree::OnRightClick(wxTreeEvent& event) {
  // PopupMenu(popup_menu, event.GetPoint());
}
