/****************************************************************************
**  Copyright (C) 2004                                                     **
**  University of Tennessee, Innovative Computing Laboratory               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/


/*******************************
				
    MetricTree.cpp	
					
********************************/

#include <iostream>

#include "MetricTree.h"
#include "CnodeTree.h"
#include "FlatTree.h"
#include "FlatRegionTree.h"
#include "PresenterApp.h"
#include "PresenterFrame.h"
#include "Metric.h"
#include "GUIData.h"
#include "CubeCollect.h"
#include "SpectrumPane.h"
#include "Browser.h"

using namespace std;
using namespace cube;


BEGIN_EVENT_TABLE(MetricTree, Tree)
  EVT_MENU    (Met_POP_ID_Des, MetricTree::OnDescr)
  EVT_MENU    (Met_POP_ID_URL, MetricTree::OnURL)
END_EVENT_TABLE()


MetricTree::MetricTree(wxWindow* parent) : Tree(parent) {
  popup_menu->Append(Met_POP_ID_URL, _T("Online Description"));
  popup_menu->Append(Met_POP_ID_Des, _T("Info"));
}

void MetricTree::OnDescr(wxCommandEvent& event) {
  PresenterFrame* frame = myApp->get_frame();
  wxString text;
  Vertex* obj = GetItemData(right_click_item)->get_obj();
  if (obj != NULL) {
    Metric* p = static_cast<Metric*>(obj);
    string descr = p->get_descr();
    text.Printf("%s\n", descr.c_str());
    wxMessageBox(text, _T("Metric Information"), wxOK, this);
    /*if (descr.find("<html>") == string::npos &&
	descr.find("<HTML>") == string::npos ) {
      text.Printf("Description: %s \n", descr.c_str());
      wxMessageBox(text, _T("Metric description"), wxOK, this);
    } else {
      if (frame->get_browser_dlg() == NULL) {
	frame->set_browser_dlg(new Browser(frame));
      }
      Browser* browser = frame->get_browser_dlg();
      browser->SetPage(descr.c_str());
      browser->set_big_size(false);
      browser->Show(true);
    }
    */
  }
}

void MetricTree::OnURL(wxCommandEvent& event) {
  PresenterFrame* frame = myApp->get_frame();
  Vertex* obj = GetItemData(right_click_item)->get_obj();
  if (obj != NULL) {
    Metric* p = static_cast<Metric*>(obj);
    string url = p->get_url();
    if (frame->get_browser_dlg() == NULL) {
      frame->set_browser_dlg(new Browser(frame));
    }
    Browser* browser = frame->get_browser_dlg();
    int ret = frame->load_working_url(browser, url);
    if (ret < 0) {
      cerr << "Error: " << url << " not accessible." << endl;
      return;
    }
    if (ret > 0) {
      browser->Show(true);
      browser->Raise();
    }
  }
}

void MetricTree::OnExpanded(wxTreeEvent& event) {
  wxTreeItemId id = event.GetItem();
  PresenterFrame* frame = myApp->get_frame();
  Vertex* obj = GetItemData(id)->get_obj();
  if (obj == NULL) {
    cerr << "Warning: metric node expanded has a NULL obj" << endl;  
    return;
  }
  const Metric* p = static_cast<Metric*>(obj);
  GUIData* guidata = frame->get_guidata();
  if (IsSelected(id)) guidata->xpnd_sel_left(); // update guidata
  string label = guidata->get_met_label(CUBE_EXCL, p);
  SetItemText(id, label, guidata->get_met_color(CUBE_EXCL, p));
  if (IsSelected(id)) {
    frame->get_mid_tree()->refresh();
  }
}

void MetricTree::OnCollapsed(wxTreeEvent& event) {
  wxTreeItemId id = event.GetItem();
  PresenterFrame* frame = myApp->get_frame();
  Vertex* obj = GetItemData(id)->get_obj();
  if (obj == NULL) {
    cerr << "Warning: metric node collapsed has a NULL obj" << endl;  
    return;
  }
  const Metric* p = static_cast<Metric*>(obj);
  GUIData* guidata = frame->get_guidata();
  if (IsSelected(id)) guidata->clps_sel_left(); // update guidata
  string label = guidata->get_met_label(CUBE_INCL, p);
  SetItemText(id, label, guidata->get_met_color(CUBE_INCL, p));
  // if it's also the selected metric
  if (IsSelected(id)) {
    frame->get_mid_tree()->refresh();
  }
}

void MetricTree::OnSelChanged(wxTreeEvent& event) {
  wxTreeItemId id = GetSelection(); // newly selected node.
  PresenterFrame* frame = myApp->get_frame();
  Vertex* obj = GetItemData(id)->get_obj();
  if (obj == NULL) {
    cerr << "Warning: metric node changed has a NULL obj" << endl;  
    return;
  }
  GUIData* guidata = frame->get_guidata();
  const Metric* met = static_cast<Metric*>(obj);
  guidata->select_metric(met);
  //  cout << "first selected metric: " << met->get_name() << endl;
  IsExpanded(id) ? guidata->xpnd_sel_left() : guidata->clps_sel_left();
  if (frame->get_mid_tree() != NULL)
    frame->get_mid_tree()->refresh();
  // refresh spectrum pane's max values if metric tree changes
  frame->get_spectrum_pane()->Refresh();
}


void MetricTree::OnRightClick(wxTreeEvent& event) {
  right_click_item = HitTest(event.GetPoint());
  PresenterFrame* frame = myApp->get_frame();
  Vertex* obj = GetItemData(right_click_item)->get_obj();
  if (obj == NULL) return;

  Metric* p = static_cast<Metric*>(obj);
  string url = p->get_url();
  string descr = p->get_descr();

  if (url.empty())  popup_menu->Enable(Met_POP_ID_URL, false);
  if (descr.empty())popup_menu->Enable(Met_POP_ID_Des, false); 
  PopupMenu(popup_menu, event.GetPoint());
  if (url.empty())  popup_menu->Enable(Met_POP_ID_URL, true);
  if (descr.empty())popup_menu->Enable(Met_POP_ID_Des, true);

  /*
  Metric* p = static_cast<Metric*>(obj);
  string url = p->get_url();
  if(!url.empty()) {//dynamically add menu item
    string text = "Goto URL";
    popup_menu->Append(Met_POP_ID_URL, text.c_str());
  }
  PopupMenu(popup_menu, event.GetPoint());
  if(!url.empty()) {//dynamically remove menu item
    popup_menu->Delete(Met_POP_ID_URL);
  }
  */
}

// called only once and it won't refresh mid&right trees!
void MetricTree::refresh () {
  if (GetChildrenCount(GetVRootItem(), false) == 0) return;
  traverse(GetVRootItem());
#ifdef CUBE_DEBUG
  cout << "metric tree is refreshed." << endl;
#endif

}

void MetricTree::traverse(wxTreeItemId root) {
  long cookie;
  PresenterFrame* frame = myApp->get_frame();
  GUIData* guidata = frame->get_guidata();

  Vertex* obj = GetItemData(root)->get_obj();
  
  // update the root's label.
  if (obj != NULL) {// not virtual root
    Metric* p = static_cast<Metric*>(obj);
    int mode = IsExpanded(root) ? CUBE_EXCL : CUBE_INCL;
    string label = guidata->get_met_label(mode, p);
    SetItemText(root, label, guidata->get_met_color(mode, p));
  }
  if (GetChildrenCount(root) == 0) {
    return;
  }
  wxTreeItemId child = GetFirstChild(root, cookie); 
  // then update child label recursively 
  traverse(child);
  while (child = GetNextChild(root, cookie)) {
    traverse(child);
  }
  return;
}
