/****************************************************************************
**  Copyright (C) 2004                                                     **
**  University of Tennessee, Innovative Computing Laboratory               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

/*******************************
				
     MidTree.cpp	
					
********************************/

#include <iostream>
#include <fstream>

#ifdef _WIN32
 #include <direct.h>
 #define getcwd _getcwd
#else
 #include <unistd.h>
#endif

#include "Tree.h"
#include "MidTree.h"
#include "PresenterFrame.h"
#include "PresenterApp.h"
#include "SourceCodeDlg.h"

#include "Module.h"
#include "Region.h"

using namespace std;
using namespace cube;

BEGIN_EVENT_TABLE(MidTree, Tree)
  EVT_MENU       (MID_POP_ID_Des,   MidTree::OnDes)
  EVT_MENU       (MID_POP_ID_Loc,   MidTree::OnLoc)
  EVT_MENU       (MID_POP_ID_Code,  MidTree::OnSourceCode)
  EVT_MENU       (MID_POP_ID_URL,   MidTree::OnURL)
END_EVENT_TABLE()


MidTree::MidTree (wxWindow* parent) : Tree(parent){
  default_path = ".";
  char dirname[128];
  if (getcwd(dirname, 128) != NULL) {
    default_path = dirname;
  }
  
  popup_menu->Append(MID_POP_ID_Loc,   _T("Location"));
  popup_menu->Append(MID_POP_ID_Code,  _T("Source code"));  
  popup_menu->Append(MID_POP_ID_URL,   _T("Online Description"));
  popup_menu->Append(MID_POP_ID_Des,   _T("Info"));
}

/* Check whether it's possible to read out that line.
   
   if module is not existing, return empty string;
   if the line is empty, return "empty line";
   if the line is not existing, return "N/A".
*/
string MidTree::get_line(string path, string fname, int line) {
  char code[1024];
  if (path.empty()) {
    // if empty, set it as last searching path.
    path = default_path; // "."
  }
  // if path is not empty, we assume the path is correct.
  // Users have to modify their cube files to make it work.
  
  path += wxFILE_SEP_PATH;
  path += fname;
  // cout << "module name: " << path << endl;
  ifstream in (path.c_str());
  string ret;
  if (!in) {
    wxString msg;
    msg.Printf("Module %s not found!\n\nClick Ok to try another directory?",
	       path.c_str());
    wxMessageDialog dlg(this, msg, "", wxOK | wxCANCEL | wxICON_ERROR);
    if (dlg.ShowModal() == wxID_OK){
      wxString dir=::wxDirSelector(_T("Select search directory"),
				    _T(default_path.c_str()));
      if ( !dir )  return "";
      default_path = dir.c_str(); // set it as default path
      ret = get_line(dir.c_str(), fname, line); // recursive call
      return ret;
    }
    return ""; // User cancelled, return empty string.
  }

  suc_path = path; 

  if (line <= 0) return "N/A";

  int counter = 0;
  while (in.getline(code, 1024)) { // search for that line
    counter++;
    if (counter == line) break;
  }
  if (counter == line) {
    in.close();
    if (string(code).empty()) return "  empty Line";
    else return string(code);
  }
  in.close();
  return "N/A"; // line not found
}

/* All the file's souce code will be shown in dialog.
   if highlighting one line, just give begn_ln==end_ln    
   Dependent on suc_path.
*/
void MidTree::display_region_code(int begn_ln, int end_ln) {
  PresenterFrame* frame = myApp->get_frame();
  ifstream in (suc_path.c_str());

  if (!in) {
    cerr << "File open error: " << suc_path << endl;
    return; // do nothing about it.
  }
  in.close();

  if (begn_ln < 0 && end_ln < 0) {
    wxMessageBox("Position undefined!", "Error", wxICON_ERROR, this);
    // begn_ln = end_ln = 1;
  } else if (begn_ln < 0) {
    begn_ln = end_ln;
    cerr << "Warning: regions with begin line number < 0" << endl;
  } else if (end_ln < 0) {
    cerr << "Warning: regions with end line number < 0" << endl;
    end_ln = begn_ln;
  }

  frame->get_source_dlg()->show_module(suc_path);
  frame->get_source_dlg()->highlight(begn_ln, end_ln);
  frame->get_source_dlg()->show_line(begn_ln);
  frame->get_source_dlg()->Show(true);
}

string MidTree::region_ln_label(int bln, int eln) {
  wxString line_text;
  if (bln > 0 && eln > 0) {
    line_text.Printf("Lines: %d-%d", bln, eln);
  } else if (bln > 0 && eln <= 0) {
    line_text.Printf("Lines: %d-?", bln);
  } else if (bln <= 0 && eln > 0) {
    line_text.Printf("Lines: ?-%d", eln);
  } else {
    line_text.Printf("Lines: Undefined"); // nothing available
  }  
  return line_text.c_str();
}

// only used for regions
void MidTree::show_descr_dlg4reg(const Region* r) {
  wxString text; 
  if (r == NULL) return;
  wxDialog* dlg = new wxDialog(this, -1, "Region location", 
			       wxPoint(0,0), wxSize(300, 100));
  wxBoxSizer* sizer = new wxBoxSizer(wxVERTICAL);
  const Module* m = r->get_mod();
  // module name
  string fname = m->get_name();
  if (fname.empty()) fname = "Undefined";
  text = "Module name: ";
  text.Append(fname.c_str());
  sizer->Add(new wxStaticText(dlg, -1, text, wxDefaultPosition, wxSize(300, -1)),
	    1,
	    wxALL, 3);
  int ln1 = r->get_begn_ln();
  int ln2 = r->get_end_ln();
  text = region_ln_label(ln1, ln2).c_str();
  sizer->Add(new wxStaticText(dlg, -1, text, wxDefaultPosition, wxSize(300, -1)),
	    1,wxALL, 3);
  /*
  // begin line number
  int ln1 = r->get_begn_ln();
  text.Printf("Begin line number: %d", ln1);
  sizer->Add(new wxStaticText(dlg, -1, text, wxDefaultPosition, wxSize(300, -1)),
	    1,wxALL, 3);
  // end line number
  int ln2 = r->get_end_ln();
  text.Printf("End line number: %d", ln2);
  sizer->Add(new wxStaticText(dlg, -1, text, wxDefaultPosition, wxSize(300, -1)),
	     1,wxALL, 3); 
  */

  // "OK" button
  sizer->Add(new wxButton(dlg, wxID_OK, _("&Ok")), 1, 
	    wxALIGN_CENTRE_HORIZONTAL, 3);
  
  dlg->SetSizer(sizer);
  dlg->Centre(wxBOTH);
  dlg->ShowModal();
  delete dlg;
}


