/****************************************************************************
**  Copyright (C) 2004                                                     **
**  University of Tennessee, Innovative Computing Laboratory               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

/***************************************
					
       MidTree class
     
     
***************************************/


#ifndef CUBE_MID_TREE_H
#define CUBE_MID_TREE_H

#include <string>

#include "wx/wxprec.h"
#include "wx/wx.h"
#include "wx/treectrl.h"
#include "Tree.h"

namespace cube {
  class Region;

  enum MID_PopMenu { 
    MID_POP_ID_Des,
    MID_POP_ID_Loc,
    MID_POP_ID_Code,
    MID_POP_ID_URL,       // all above are shared
    CALLTREE_POP_ID_Loc,  // specific for call-tree
    CALLTREE_POP_ID_Code  // specific for call-tree
  };
  
  class MidTree : public Tree {
  public:
    MidTree (wxWindow* parent); 
    
    std::string get_line(std::string pathname, std::string filename, int line);
    void display_region_code (int begn_ln, int end_ln);
    static std::string region_ln_label(int begn_ln, int end_eln);
    void show_descr_dlg4reg(const Region* r);

    // triggered by a change in metric tree
    virtual void  refresh () = 0;

    // context menu handler
    virtual void   OnDes           (wxCommandEvent& event) = 0;
    virtual void   OnLoc           (wxCommandEvent& event) = 0;
    virtual void   OnSourceCode    (wxCommandEvent& event) = 0;        
    virtual void   OnURL           (wxCommandEvent& event) = 0;


    DECLARE_EVENT_TABLE()
      
  private:     

    virtual void   traverse(wxTreeItemId root) = 0;    

    std::string default_path; // default directory
    std::string suc_path;     // correct path which passed get_line()
  };
  
}

#endif
