/****************************************************************************
**  Copyright (C) 2004                                                     **
**  University of Tennessee, Innovative Computing Laboratory               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/


/*******************************
				
     Tree.cpp	
					
********************************/

#include <iostream>
#include "Tree.h"
#include "PresenterApp.h"

using namespace std;
using namespace cube;

BEGIN_EVENT_TABLE(Tree, wxTreeCtrl)
EVT_TREE_SEL_CHANGED      (-1, Tree::OnSelChanged)
EVT_TREE_ITEM_EXPANDED    (-1, Tree::OnExpanded)
EVT_TREE_ITEM_COLLAPSED   (-1, Tree::OnCollapsed)
EVT_TREE_ITEM_RIGHT_CLICK (-1, Tree::OnRightClick)
END_EVENT_TABLE()


Tree::Tree(wxWindow* parent): 
  wxTreeCtrl(parent, -1, wxDefaultPosition, wxDefaultSize,
	     wxTR_DEFAULT_STYLE|wxSUNKEN_BORDER|wxTR_HIDE_ROOT|
	     wxTR_HAS_BUTTONS) {
  myApp = static_cast<PresenterApp*>(wxTheApp);
  SetImageList(myApp->get_image_list());
  popup_menu = new wxMenu;
  // virtual invisible root.
  wxTreeItemId id = wxTreeCtrl::AddRoot(_T(""));
  SetItemData(id, new NodeData(NULL));//not null pointer pointing to a NULL.
  virtual_root = id;
  // SetIndent(10);
}

wxTreeItemId Tree::AddRoot(string label, int color, NodeData* nd) {
  wxTreeItemId id = AppendItem(virtual_root, label, color, nd);
  return id;
}

wxTreeItemId Tree::
AppendItem(wxTreeItemId parent, string label, int color, NodeData* nd) {
  wxTreeItemId id = wxTreeCtrl::
    AppendItem(parent, wxString(label.c_str()), -1, -1, nd);
  SetItemImage(id, color);
  return id;
}

void Tree::SetItemText(wxTreeItemId id, string label, int color) {
  wxTreeCtrl::SetItemText(id, wxString(label.c_str()));
  SetItemImage(id, color);
}

void Tree::SetItemData(wxTreeItemId id, NodeData* nd) {
  wxTreeCtrl::SetItemData(id, nd);
}

const NodeData* Tree::GetItemData(wxTreeItemId id) {
  return static_cast<NodeData*>(wxTreeCtrl::GetItemData(id));
}


