/****************************************************************************
**  Copyright (C) 2004                                                     **
**  University of Tennessee, Innovative Computing Laboratory               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/


/***************************************
					
       Tree class
     
     an abstract class
***************************************/


#ifndef CUBE_TREE_H
#define CUBE_TREE_H

#include <string>

#include "wx/wxprec.h"
#include "wx/wx.h"
#include "wx/treectrl.h"

#include "Vertex.h"

namespace cube {
  class PresenterApp;

  // struct associated with a tree item
  class NodeData : public wxTreeItemData {
  public:
    NodeData(Vertex* pt) : obj(pt) {} 
    Vertex* get_obj() const { return obj; }

  private:
    Vertex* obj;
  };

  // each dimension has such a tree (presented as a "forest")
  class Tree : public wxTreeCtrl {
  public:
    Tree (wxWindow* parent);
    ~Tree() {}
    
    wxTreeItemId  AddRoot     (std::string label, int color, NodeData* nd = NULL);
    wxTreeItemId  AppendItem  (wxTreeItemId parentid, std::string label, 
			       int color, NodeData* nd);
    void SetItemText (wxTreeItemId id, std::string label, int color);
    void SetItemData (wxTreeItemId id, NodeData* nd);
    const NodeData* GetItemData (wxTreeItemId id);
    
    wxTreeItemId GetVRootItem()    { return virtual_root; } //"root" of forest
    
    DECLARE_EVENT_TABLE()
      
    wxMenu*      popup_menu;
    wxTreeItemId right_click_item; 
    PresenterApp* myApp;    

  private:
    
    virtual void OnSelChanged  (wxTreeEvent& event) = 0;
    virtual void OnExpanded    (wxTreeEvent& event) = 0;
    virtual void OnCollapsed   (wxTreeEvent& event) = 0;
    virtual void OnRightClick  (wxTreeEvent& event) = 0;
    
    wxTreeItemId virtual_root;

  };
  
}

#endif
