/****************************************************************************
**  Copyright (C) 2004                                                     **
**  University of Tennessee, Innovative Computing Laboratory               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

/*******************************************
     Machine.cpp

********************************************/

#include "Machine.h"
#include "Node.h"

#include "CUBException.h"

using namespace std;
using namespace cube;

const Node* cube::Machine::get_child(int i)  const { 
  if (i < 0 || i >= nodev.size()) 
    throw CUBException("machine vector out of range");
  return nodev[i]; 
}

void Machine::writeXML(std::ostream& out, int& loc_id,
		       map<const Location*, int>& loc_mapping) const {
  int level;

  level = get_level();
  loc_mapping[this] = loc_id;
  out << indent() << "<machine locid=\"" << loc_id++ << "\">" << endl; 
  // out << "\" locId=\"" << loc_id++ << "\">"<<endl;
  set_level(level+1);
  out << indent() << "<name>" << get_name() << "</name>" << endl;
  for(int i = 0; i < num_children() ; i++) {
    const Node* node = get_child(i);
    node->set_level(level+1);
    node->writeXML(out, loc_id, loc_mapping);
  }

  set_level(level);
  out << indent() << "</machine>" << endl;

}

