/****************************************************************************
**  Copyright (C) 2004                                                     **
**  University of Tennessee, Innovative Computing Laboratory               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#ifndef CUBE_MACHINE_H
#define CUBE_MACHINE_H   

/*
 *----------------------------------------------------------------------------
 *
 * class Machine
 *
 *----------------------------------------------------------------------------
 */

#include <vector>
#include <string>
#include <map>

#include "Location.h"
#include "Node.h"

namespace cube {
  class Machine: public Location {
    
  public:
    Machine(std::string name): Location(name) {}
    
    void add_node(Node* node)    { nodev.push_back(node); }
    int  num_nodes()   const     { return num_children(); }
    
    const Node* get_child(int i) const;
    int num_children() const     { return nodev.size(); }

    const std::vector<Node*>& get_children() const { return nodev;}
    void writeXML(std::ostream& out, int& loc_id,
		  std::map<const Location*, int>& loc_mapping) const;  

  private:
    
    std::vector<Node*> nodev;
  };
  
  inline bool operator== (const Machine& a, const Machine& b) {
    return a.get_name() == b.get_name();
  }
  
}

#endif
