/****************************************************************************
**  Copyright (C) 2004                                                     **
**  University of Tennessee, Innovative Computing Laboratory               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/


/************************************************
             CubeMatrix.cpp


 ************************************************/

#include "Matrix.h"

using namespace std;
using namespace cube;

double Matrix::sum(){
  double val = 0.0;
  Matrix::const_iterator i;
  map<const Thread*, double>::const_iterator j;
  for (i = begin(); i != end(); i++) {
    for (j = i->second.begin(); j != i->second.end(); j++) {
      val += j->second;
    }
  }  
  
  return val;
}


/* helper functions */

namespace cube {

/* matrix - matrix */
Matrix operator-(Matrix& mat1, Matrix& mat2) {
  Cnode *r1, *r2; // row index
  Matrix ret;
  Matrix::const_iterator i1, i2;
  
  for (i1 = mat1.begin(), i2 = mat2.begin(); i1 != mat1.end() || i2 != mat2.end(); /* empty */) {
    if (i1 == mat1.end()) r1 = NULL;
    else r1 = const_cast<Cnode*>(i1->first);
    if (i2 == mat2.end()) r2 = NULL;
    else r2 = const_cast<Cnode*>(i2->first);
    if (r1 == r2) {
      // compute one row
      ret[r1] = mat1[r1] - mat2[r2];
      i1++;
      i2++;
    } else if (r1 < r2 && r1 != NULL || r2 == NULL) {
      ret[r1] = mat1[r1];
      i1++;
    } else {
      ret[r2] = -mat2[r2];
      i2++;
    }
  } // end of for loop
  
  return ret;
}

Row operator-(Row& map1, Row& map2) {
  Row ret ;
  Thread *c1, *c2; // column index
  Row::const_iterator i1, i2;
  // input should not be empty map
  for (i1 = map1.begin(), i2 = map2.begin();
       i1 != map1.end() || i2 != map2.end(); ) {
    if (i1 == map1.end()) c1 = NULL;
    else c1 = const_cast<Thread*>(i1->first);
    if (i2 == map2.end()) c2 = NULL;
    else c2 = const_cast<Thread*>(i2->first);
    if (c1 == c2) {
      // compute one row
      ret[c1] = map1[c1] - map2[c2];
      if (ret[c1] < 0 && -ret[c1] < 1.0e-6) ret[c1] = 0;
      // if two elements are positive, but their difference
      // is negative, then assign a "0" to it.
      /*if (ret[c1] < 0 && map1[c1] >=0 && map2[c2] >= 0) {
	ret[c1] = 0.0;
      }
      */
      i1++;
      i2++;
    } else if (c1 < c2 && c1 != NULL || c2 == NULL) {
      ret[c1] = map1[c1];
      i1++;
    } else {
      ret[c2] = -map2[c2];
      i2++;
    }    
  } // end of for loop
  
  return ret;
}

/* -vector, argument value not changed. */
Row operator-(Row& map1) {
  Row ret ;
  Thread* c; // column index
  Row::const_iterator i;
  
  for (i = map1.begin(); i != map1.end(); i++) {
    c = const_cast<Thread*>(i->first);
    ret[c] = -map1[c];
  } // end of for loop
  
  return ret;
}

// vector + operator
Row operator+(Row& map1, Row& map2) {
  Row ret ;
  Thread *c1, *c2; // column index
  Row::const_iterator i1, i2;
  // input should not be empty map
  for (i1 = map1.begin(), i2 = map2.begin();
       i1 != map1.end() || i2 != map2.end(); ) {
    if (i1 == map1.end()) c1 = NULL;
    else c1 = const_cast<Thread*>(i1->first);
    if (i2 == map2.end()) c2 = NULL;
    else c2 = const_cast<Thread*>(i2->first);
    if (c1 == c2) {
      // compute one row
      ret[c1] = map1[c1] + map2[c2];
      i1++;
      i2++;
    } else if (c1 < c2 && c1 != NULL || c2 == NULL) {
      ret[c1] = map1[c1];
      i1++;
    } else {
      ret[c2] = map2[c2];
      i2++;
    }    
  } // end of for loop
  
  return ret;
}

// matrix + operator
Matrix operator+(Matrix& mat1, Matrix& mat2) {
  Cnode *r1, *r2; // row index
  Matrix ret;
  Matrix::const_iterator i1, i2;
  
  for (i1 = mat1.begin(), i2 = mat2.begin(); 
       i1 != mat1.end() || i2 != mat2.end(); /* empty */) {
    if (i1 == mat1.end()) r1 = NULL;
    else r1 = const_cast<Cnode*>(i1->first);
    if (i2 == mat2.end()) r2 = NULL;
    else r2 = const_cast<Cnode*>(i2->first);
    if (r1 == r2) {
      // compute one row
      ret[r1] = mat1[r1] + mat2[r2];
      i1++;
      i2++;
    } else if (r1 < r2 && r1 != NULL || r2 == NULL) {
      ret[r1] = mat1[r1];
      i1++;
    } else {
      ret[r2] = mat2[r2];
      i2++;
    }
  } // end of for loop
  
  return ret;

}

// matrix / operator
Matrix operator/(Matrix& mat, double d) {
  Matrix ret;

  Matrix::const_iterator i;
  Row::const_iterator j;
  for (i = mat.begin(); i != mat.end(); i++) {
    for (j = i->second.begin(); j != i->second.end(); j++) {
      ret[i->first][j->first] = j->second/d;
    }
  }  
  return ret;
}

/* print out the matrix */
ostream& operator<<(ostream& out, Matrix& matrix) {
  Matrix::const_iterator i;
  Row::const_iterator j;
  out << "matrix: " << endl;
  for (i = matrix.begin(); i != matrix.end(); i++) {
    for (j = i->second.begin(); j != i->second.end(); j++) {
      out << j->second << "  ";
    }
    out << endl;
  }  
  
  return out;
}

}
