/****************************************************************************
**  Copyright (C) 2004                                                     **
**  University of Tennessee, Innovative Computing Laboratory               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/


#ifndef CUBE_MATRIX_H
#define CUBE_MATRIX_H   

/*
 *----------------------------------------------------------------------------
 *
 * class CubeMatrix (sparse matrix implemented with C++ map)
 *
 +
 *----------------------------------------------------------------------------
 */

#include <iostream>
#include <map>

namespace cube {
  class Cnode;
  class Thread;

  typedef std::map<const Thread*, double> Row;  
  typedef std::map<const Cnode*, Row> Mat2D;
  
  class Matrix: public Mat2D {  
  public:
    
    Matrix() : Mat2D(){}
    Matrix(const Matrix& mat) : Mat2D(mat) {}
    Matrix(const Mat2D& mat) : Mat2D(mat) {}

    double sum();

  private:
    
  };
  
  // non-member functions
  
  // matrix - operator
  Matrix operator- (Matrix& mat1, Matrix& mat2);
  Row operator-    (Row& row1, Row& row2);
  Row operator-    (Row& map); // unary -
  
  // matrix + operator
  Matrix operator+ (const Matrix& mat1, const Matrix& mat2);
  Row operator+    (const Row& row1, const Row& row2);

  // matrix / operator
  Matrix operator/ (const Matrix& mat, double d);
  
  // (FOR DEBUG) print out the matrix
  std::ostream& operator<< (std::ostream& out, Matrix& matrix);
  
}

#endif
