/****************************************************************************
**  Copyright (C) 2004                                                     **
**  University of Tennessee, Innovative Computing Laboratory               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

/*******************************************
     Metric.cpp

********************************************/

#include "Metric.h"
#include "CUBException.h"

using namespace std;
using namespace cube;



const Metric* Metric::get_child(int i) const{ 
  if (i < 0 || i >= childv.size()) {
    throw CUBException("metric vector out of range");
  }  
  return childv[i]; 
}


void Metric::writeXML (ostream& out, int& id, 
		       map<const Metric*, int>& met_mapping) const {
  int level;
  level = get_level();
  met_mapping[this] = id;
  out << indent() << "<metric id=\"" << id++ << "\">" << endl;
  set_level(level+1);
  out << indent() << "<name>" << get_name() << "</name>" << endl;
  out << indent() << "<uom>"  << get_uom() << "</uom>" << endl;
  out << indent() << "<url>" << get_url() << "</url>" << endl;
  out << indent() << "<descr>" 
      << "<![CDATA["
      << get_descr()
      << "]]>"
      << "</descr>" << endl;
  for(int i = 0; i < num_children() ; i++) {
    const Metric* p = get_child(i);
    p->set_level(level+1);
    p->writeXML(out, id, met_mapping);
  }
  set_level(level);
  out << indent() << "</metric>" << endl;

}

