/****************************************************************************
**  Copyright (C) 2004                                                     **
**  University of Tennessee, Innovative Computing Laboratory               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/


#ifndef CUBE_METRIC_H
#define CUBE_METRIC_H   

/*
 *----------------------------------------------------------------------------
 *
 * class Metric
 *
 *----------------------------------------------------------------------------
 */

#include <vector>
#include <iostream>
#include <map>

#include "Vertex.h"

namespace cube {
  
  class Metric: public Vertex {

  public:
    
    Metric(std::string name, std::string uom, std:: string url, std::string descr, Metric* parent) : name(name), uom(uom), url(url), descr(descr), parent(parent) {} 
    
    void add_child(Metric* child)               { childv.push_back(child); }
    int num_children() const                    { return childv.size(); }
    const Metric* get_child(int i) const;
    
    const std::vector<Metric*>& get_children() const { return childv;}
    
    const Metric* get_parent()     const             { return parent; }
    const Metric* get_parent()                       { return parent; }
    std::string get_name()         const             { return name; }
    std::string get_uom()          const             { return uom; }
    std::string get_descr()        const             { return descr; }
    std::string get_url()          const             { return url; }

    void writeXML(std::ostream& out, int& id,
		  std::map<const Metric*, int>& met_mapping) const;

  private:

    std::string name;
    std::string uom;
    std::string descr;
    std::string url;

    std::vector<Metric* > childv;
    Metric* parent;
    
  };
  
  inline bool operator== (const Metric& a, const Metric& b) {
    return a.get_name() == b.get_name();
  }
}



#endif









































