/****************************************************************************
**  Copyright (C) 2004                                                     **
**  University of Tennessee, Innovative Computing Laboratory               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#ifndef CUBE_MODULE_H
#define CUBE_MODULE_H   

/*
 *----------------------------------------------------------------------------
 *
 * class Module
 *
 *----------------------------------------------------------------------------
 */

#include <vector>

#include "Vertex.h"
#include "Region.h" 

namespace cube {
  
  class Module: public Vertex {
    
  public:    
    Module(std::string name) : name(name) { };
    
    std::string get_name() const         { return name; }

    void  add_region(Region* region)     { regionv.push_back(region); }
    int   num_regions()     const         { return num_children(); }
    const Region* get_child(int i) const;
    int   num_children()     const         { return regionv.size(); }
    void  sort_regn();
    void  writeXML(std::ostream& out, int& id) const;

  private:
    std::string name;
    std::vector<Region*> regionv;
  };

  inline bool operator== (const Module& a, const Module& b) {
    return a.get_name() == b.get_name();
  }
  inline bool operator<  (const Module& a, const Module& b) {
    return a.get_name() < b.get_name();
  }
}

#endif
