/****************************************************************************
**  Copyright (C) 2004                                                     **
**  University of Tennessee, Innovative Computing Laboratory               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/


/*******************************************
     Node.cpp

********************************************/

#include <algorithm>

#include "Node.h"
#include "CUBException.h"


using namespace std;
using namespace cube;

const Process* Node::get_child(int i) const {
  if (i < 0 || i >= processv.size()) 
    throw CUBException("node vector out of range");

  return processv[i]; 
}

void Node::sort_proc() {
  sort(processv.begin(), processv.end(), GenLess<Process*>()); 
}

void Node::writeXML(ostream& out, int& loc_id, 
		    map<const Location*, int>& loc_mapping) const{
  int level;
  
  level = get_level();
  loc_mapping[this] = loc_id;
  out << indent() << "<node locid=\"" << loc_id++ << "\">" << endl; 

  set_level(level+1);
  out << indent() << "<name>" << get_name() << "</name>\n";
  // out << node.indent() << "<cpuNum>" << node.get_cpu_num() << "</cpuNum>\n";
  
  for(int i = 0; i < num_children() ; i++) {
    const Process* p = get_child(i);
    p->set_level(level+1);
    p->writeXML(out, loc_id, loc_mapping);
  }

  set_level(level);
  out << indent() << "</node>\n";
}
