/****************************************************************************
**  Copyright (C) 2004                                                     **
**  University of Tennessee, Innovative Computing Laboratory               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#ifndef CUBE_NODE_H
#define CUBE_NODE_H   

/*
 *----------------------------------------------------------------------------
 *
 * class Node 
 *
 *----------------------------------------------------------------------------
 */

#include <iostream>
#include <vector>
#include <map>

#include "Location.h"
#include "Process.h"


namespace cube {

  class Machine;

  class Node: public Location {
  public:
    
    Node(std::string name, Machine* mach) : Location(name), mach(mach) {}
    
    void add_proc(Process* proc)     { processv.push_back(proc); }
    int  num_procs() const           { return num_children(); }

    const Machine* get_mach() const  { return mach; }
    const Process* get_child(int i) const; 
    int   num_children()      const  { return processv.size(); }

    const std::vector<Process*>& get_children() const { return processv;}
    void sort_proc(); // sort processes by process rank
    void writeXML(std::ostream& out, int& loc_id,
		  std::map<const Location*, int>& loc_mapping) const; 

  private:
  
    Machine* mach;
    std::vector<Process* > processv;    
  };
  
  inline bool operator== (const Node& a, const Node& b) {
    return a.get_name() == b.get_name();
  }
  
}

#endif
