/****************************************************************************
**  Copyright (C) 2004                                                     **
**  University of Tennessee, Innovative Computing Laboratory               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

/*******************************************
     Process.cpp

********************************************/

#include "Process.h"
#include "CUBException.h"

using namespace std;
using namespace cube;

const Thread* Process::get_child(int i) const {
  if (i < 0 || i >= threadv.size()) 
    throw CUBException("process vector out of range");
  return threadv[i]; 
}

void Process::writeXML(ostream& out, int& loc_id,
		       map<const Location*, int>& loc_mapping) const{
  int level;
  
  level = get_level();
  loc_mapping[this] = loc_id;
  out << indent() << "<process locid=\"" << loc_id++ << "\">" << endl; 
  //out << "\" locId=\"" << process.get_locid()<< "\">"<<endl;
  set_level(level+1);
  out << indent() << "<name>" << get_name() << "</name>" << endl;
  out << indent() << "<rank>" << get_rank() << "</rank>" << endl;
  for(int i = 0; i < num_children() ; i++) {
    const Thread* thrd = get_child(i);
    thrd->set_level(level+1);
    thrd->writeXML(out, loc_id, loc_mapping);
  }

  set_level(level);
  out << indent() << "</process>" << endl;
}
