/****************************************************************************
**  Copyright (C) 2004                                                     **
**  University of Tennessee, Innovative Computing Laboratory               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/


#ifndef CUBE_PROCESS_H
#define CUBE_PROCESS_H   

/*
 *----------------------------------------------------------------------------
 *
 * class Process
 *
 *----------------------------------------------------------------------------
 */

#include <vector>
#include <iostream>
#include <map>

#include "Location.h"
#include "Thread.h"

namespace cube {

  class Node;
  
  class Process: public Location {
    
  public:
    Process(std::string name, int rank, Node* node): Location(name), rank(rank), node(node) {}
    
    void add_thrd(Thread* thrd)       { threadv.push_back(thrd); }
    int  num_thrds()        const     { return num_children() ; }
    
    const Node* get_node() const      { return node; }
    int get_rank()         const      { return rank; }
    int num_children()     const      { return threadv.size(); }
    const Thread* get_child(int i) const;
 
    const std::vector<Thread*>& get_children() const { return threadv; }
    void writeXML(std::ostream& out, int& loc_id,
		  std::map<const Location*, int>& loc_mapping) const; 
    
  private:
    
    Node* node;
    int rank;
    std::vector<Thread*> threadv;    
  };

  inline bool operator== (const Process& a, const Process& b) {
    return a.get_rank() == b.get_rank();
  }
  inline bool operator< (const Process& a, const Process& b) {
    return a.get_rank() < b.get_rank();
  }

}

#endif
