/****************************************************************************
**  Copyright (C) 2004                                                     **
**  University of Tennessee, Innovative Computing Laboratory               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/


/*******************************************
     Region.cpp

********************************************/

#include "Region.h"
#include "CUBException.h"
#include "Module.h"

using namespace std;
using namespace cube;

const Callsite* Region::get_child(int i)  const {
  if (i < 0 || i >= csitev.size()) 
    throw CUBException("region vector out of range");
  return csitev[i]; 
}

void Region::add_csite(Callsite* csite)  {
  for(int i = 0; i < csitev.size(); i++) {
    // if it's defined before, don't add it any more
    if (csite == csitev[i]) return;
  }
  csitev.push_back(csite);  
}

void Region::writeXML(ostream& out, int& id, int mod_id) const{
  int level;
  
  level = get_level();

  out << indent() << "<region id=\"" << id++ << "\">" << endl;
  set_level(level+1);
  out << indent() << "<name>"
      << "<![CDATA["
      << get_name()
      << "]]>"
      << "</name>" << endl;
  out << indent() << "<mod>"   << mod_id       << "</mod>" << endl;
  out << indent() << "<begin>" << get_begn_ln()<< "</begin>" << endl;
  out << indent() << "<end>"   << get_end_ln() << "</end>" << endl;
  out << indent() << "<url>"   << get_url()    << "</url>" << endl;
  // out << indent() << "<descr>" << get_descr() << "</descr>" << endl;
  out << indent() << "<descr>" 
      << "<![CDATA["
      << get_descr()
      << "]]>"
      << "</descr>" << endl;
  set_level(level);
  out << indent() << "</region>" << endl;

}


namespace cube {
  bool operator== (const Region& a, const Region& b) {
    return a.get_name() == b.get_name() && *a.get_mod() == *b.get_mod();
  }

  // not used any more !!!
  string replace_amp(string s){
    char ret[512];
    int j = 0;
    for (int i = 0; i < s.length(); i++) {
      ret[j] = s[i];
      if (ret[j] == '&') {
	// ret[j] = ' ';
	ret[++j] = 'a'; ret[++j] = 'm'; ret[++j] = 'p'; ret[++j] = ';';
      }
      j++;
    }
    ret[j] = 0;
    return ret;
  }

}
