/****************************************************************************
**  Copyright (C) 2004                                                     **
**  University of Tennessee, Innovative Computing Laboratory               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#ifndef CUBE_REGION_H
#define CUBE_REGION_H

/*
 *----------------------------------------------------------------------------
 *
 * class Region
 *
 *----------------------------------------------------------------------------
 */

#include <vector>
#include <string>

#include "Vertex.h"
#include "Callsite.h"

namespace cube {

  class Module;

  class Region: public Vertex {
  
  public:
    
    Region(std::string name, int begln, int endln, std::string url, std::string descr, Module* mod) : 
      name(name), mod(mod), begln(begln), endln(endln), url(url), descr(descr) {}

    std::string get_name() const       { return name; }
    std::string get_descr()const       { return descr; }
    std::string get_url()  const       { return url; }

    const Module* get_mod()const       { return mod; }
    
    int  num_csites() const            { return num_children(); }
    void add_csite(Callsite* csite);
    const Callsite* get_child(int i) const;
    int  num_children() const          { return csitev.size(); }
    
    int  get_begn_ln()  const          { return begln; }
    int  get_end_ln()   const          { return endln; }

    void writeXML(std::ostream& out, int& id, int mod_id) const;
  

  private:
    
    std::string name;
    std::string url;
    std::string descr;
    
    int begln;
    int endln;
    
    Module* mod;
    std::vector<Callsite*> csitev;
  };

  bool operator== (const Region& a, const Region& b);
  inline bool operator<  (const Region& a, const Region& b) {
    return a.get_name() < b.get_name();
  }

  std::string replace_amp(std::string s);

}

#endif
