/****************************************************************************
**  Copyright (C) 2004                                                     **
**  University of Tennessee, Innovative Computing Laboratory               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

/******************************
     testing the Cube class

*******************************/

#include <iostream>
#include <string>
#include <vector>
#include <fstream>

#include "Cube.h"
#include "Space.h"
#include "CubeCollect.h"
#include "CubeCached.h"

#include "Metric.h"
#include "Cnode.h"
#include "Process.h"
#include "Node.h"
#include "Thread.h"

using namespace std;
using namespace cube;

/* Debug: only used for Metric, Cnode */
template<class T>
void traverse_print(T root) {
  cout << root->get_id() << endl;
  for(int i = 0; i < root->num_children(); i++) {
    traverse_print<T>((T) root->get_child(i));
  }
}

// example code.
int main(int argc, char* argv[]) {
  const Metric *met0, *met1, *met2;
  const Module* mod;
  const Region *regn0, *regn1, *regn2;
  const Callsite *csite0, *csite1, *csite2;
  const Cnode  *cnode0, *cnode1, *cnode2;
  const Machine* mach;
  const Node* node;
  const Process* proc;
  const Thread *thrd0, *thrd1;

  Cube cube;

  // Specify mirrors (optional)
  cube.def_mirror("http://icl.cs.utk.edu/software/kojak/");
  cube.def_mirror("http://www.fz-juelich.de/zam/kojak/");

  // Specify information related to the file (optional)
  cube.def_attr("experiment time", "November 1st, 2004");
  cube.def_attr("description", "a simple example");

  // Build metric tree 
  met0 = cube.def_met("Time", "sec", 
		      "@mirror@patterns-2.1.html#execution", 
		      "root node", NULL); // using mirror
  met1 = cube.def_met("User time", "sec", 
                      "http://www.cs.utk.edu/usr.html", 
                      "2nd level", met0); // withoug using mirror
  met2 = cube.def_met("System time", "sec", 
                       "http://www.cs.utk.edu/sys.html", 
                      "2nd level", met0); // withoug using mirror

  // Build call tree
  mod    = cube.def_module("/ICL/CUBE/example.c");
  regn0  = cube.def_region("main", 21, 100, "", "1st level", mod);
  regn1  = cube.def_region("foo", 1, 10, "", "2nd level", mod);
  regn2  = cube.def_region("bar", 11, 20, "", "2nd level", mod);
  csite0 = cube.def_csite(mod, 21, regn0);
  csite1 = cube.def_csite(mod, 60, regn1);
  csite2 = cube.def_csite(mod, 80, regn2);
  cnode0 = cube.def_cnode(csite0, NULL);
  cnode1 = cube.def_cnode(csite1, cnode0);
  cnode2 = cube.def_cnode(csite2, cnode0);

  // Build location tree
  mach  = cube.def_mach("MSC");
  node  = cube.def_node("Athena", mach);
  proc  = cube.def_proc("Process 0", 0, node);
  thrd0 = cube.def_thrd("Thread 0", 0, proc);
  thrd1 = cube.def_thrd("Thread 1", 1, proc);

  // Severity mapping
  cube.set_sev(met0, cnode0, thrd0, 4);
  cube.set_sev(met0, cnode0, thrd1, 4);
  cube.set_sev(met0, cnode1, thrd0, 4);
  cube.add_sev(met0, cnode1, thrd1, 4);
  cube.add_sev(met0, cnode2, thrd0, 4);
  cube.add_sev(met0, cnode2, thrd1, 4);
  cube.set_sev(met1, cnode0, thrd0, 1);
  cube.set_sev(met1, cnode0, thrd1, 1);
  cube.set_sev(met1, cnode1, thrd0, 1);
  cube.add_sev(met1, cnode1, thrd1, 1);
  cube.add_sev(met1, cnode2, thrd0, 1);
  cube.add_sev(met1, cnode2, thrd1, 1);
  cube.set_sev(met2, cnode0, thrd0, 1);
  cube.set_sev(met2, cnode0, thrd1, 1);
  cube.set_sev(met2, cnode1, thrd0, 1);
  cube.add_sev(met2, cnode1, thrd1, 1);
  cube.add_sev(met2, cnode2, thrd0, 1);
  cube.add_sev(met2, cnode2, thrd1, 1);
  
  // Output to a cube file
  ofstream out;
  out.open("example.cube");
  out << cube;

}

// int main(int argc, char* argv[]) {
//   Cube cube;  
//   // test metrics dimension
//   cube.def_attr("experiment location", "icl utk");
//   cube.def_attr("info", "a simple cube test");
//   // cube.set_attr("CUBE_CT_AGGR", "MAX");
//   cube.def_mirror("http://icl.cs.utk.edu");
//   cube.def_mirror("http://www.cs.utk.edu");
//   cube.def_mirror("http://www.cs.ubc.ca");

//   const Metric* met0 = cube.def_met("Time", "sec", "http://www.cs.utk.edu", "root", NULL);
//   const Metric* met1 = cube.def_met("User time", "sec", "", "2nd level", met0);
//   const Metric* met2 = cube.def_met("System time", "sec", "", "2nd level", met0);

//   // test callnode dimension
//   const Module* mod = cube.def_module("main.cpp");
//   const Region* r0 = cube.def_region("main&", 100, 200,"http://icl.cs.utk.edu",
// 				     "1st level", mod);
//   const Region* r1 = cube.def_region("foo", 20, 80, "", "2nd level", mod);
//   const Region* r2 = cube.def_region("bar", 200, 300, "", "2rd level", mod);

//   const Callsite* cs0 = cube.def_csite(mod, 3, r0);
//   const Callsite* cs1 = cube.def_csite(mod, 150, r1);
//   const Callsite* cs2 = cube.def_csite(mod, 160, r2);
//   const Cnode* cn0 = cube.def_cnode(cs0, NULL);
//   const Cnode* cn1 = cube.def_cnode(cs1, cn0);
//   const Cnode* cn2 = cube.def_cnode(cs2, cn0);
//   // for 2nd call-tree
//   const Cnode* cn3 = cube.def_cnode(cs1, NULL);
//   const Cnode* cn4 = cube.def_cnode(cs2, cn3);
//   // for call-tree without line number
//   /*const Cnode* cn0 = cube.def_cnode(r0, NULL);
//   const Cnode* cn1 = cube.def_cnode(r1, cn0);
//   const Cnode* cn2 = cube.def_cnode(r2, cn0);*/

//   // test location dimension
//   const Machine* mach = cube.def_mach("linux cluster");
//   const Node* node = cube.def_node("athena",  mach);
//   const Process* proc = cube.def_proc("Process 0", 0, node);
//   const Thread* thrd0 = cube.def_thrd("Thread 0", 0, proc);
//   const Thread* thrd1 = cube.def_thrd("Thread 1", 1, proc);

//   // test severity matrix
//   cube.set_sev(met0, cn0, thrd0, 4);
//   cube.set_sev(met0, cn0, thrd1, 4);
//   cube.set_sev(met0, cn1, thrd0, 4);
//   cube.set_sev(met0, cn1, thrd1, 4);
//   cube.set_sev(met0, cn2, thrd0, 4);
//   cube.set_sev(met0, cn2, thrd1, 4);
//   cube.set_sev(met1, cn0, thrd0, 1);
//   cube.set_sev(met1, cn0, thrd1, 1);
//   cube.set_sev(met1, cn1, thrd0, 1);
//   cube.set_sev(met1, cn1, thrd1, 1);
//   cube.set_sev(met1, cn2, thrd0, 1);
//   cube.set_sev(met1, cn2, thrd1, 1);
//   cube.set_sev(met2, cn0, thrd0, 1);
//   cube.set_sev(met2, cn0, thrd1, 1);
//   cube.set_sev(met2, cn1, thrd0, 1);
//   cube.set_sev(met2, cn1, thrd1, 1);
//   cube.set_sev(met2, cn2, thrd0, 1);
//   cube.set_sev(met2, cn2, thrd1, 1);
//   // set 2nd call-tree
//   cube.set_sev(met0, cn3, thrd0, 3);
//   cube.set_sev(met0, cn3, thrd1, 3);
//   cube.set_sev(met0, cn4, thrd0, 3);
//   cube.set_sev(met0, cn4, thrd1, 3);
//   cube.set_sev(met1, cn3, thrd0, 1);
//   cube.set_sev(met1, cn3, thrd1, 1);
//   cube.set_sev(met1, cn4, thrd0, 1);
//   cube.set_sev(met1, cn4, thrd1, 1);
//   cube.set_sev(met2, cn3, thrd0, 1);
//   cube.set_sev(met2, cn3, thrd1, 1);
//   cube.set_sev(met2, cn4, thrd0, 1);
//   cube.set_sev(met2, cn4, thrd1, 1);

//   // for flat-tree only
//   /*
//   cube.set_sev(met0, r0, thrd0, 4);
//   cube.set_sev(met0, r0, thrd1, 4);
//   cube.set_sev(met0, r1, thrd0, 4);
//   cube.set_sev(met0, r1, thrd1, 4);
//   cube.set_sev(met0, r2, thrd0, 4);
//   cube.set_sev(met0, r2, thrd1, 4);
//   cube.set_sev(met1, r0, thrd0, 1);
//   cube.set_sev(met1, r0, thrd1, 1);
//   cube.set_sev(met1, r1, thrd0, 1);
//   cube.set_sev(met1, r1, thrd1, 1);
//   cube.set_sev(met1, r2, thrd0, 1);
//   cube.set_sev(met1, r2, thrd1, 1);
//   cube.set_sev(met2, r0, thrd0, 1);
//   cube.set_sev(met2, r0, thrd1, 1);
//   cube.set_sev(met2, r1, thrd0, 1);
//   cube.set_sev(met2, r1, thrd1, 1);
//   cube.set_sev(met2, r2, thrd0, 1);
//   cube.set_sev(met2, r2, thrd1, 1);
//   */
//   // output to xml.
//   ofstream out;
//   out.open("a.cube", ofstream::out | ofstream::trunc);
//   cout<<"\nCreated a cube space in memory \n";
//   out << cube;
//   cout<<"Output it to XML doc: a.cube. \n\n";
//   out.close();


//   // read xml in.
//   Cube cube22;
//   ifstream in ("a.cube");
//   if (!in) cout << "open a.cube Error \n";
//   in >> cube22;
//   in.close();
//   // output to xml again
//   ofstream out2;
//   out2.open("b.cube", ofstream::out | ofstream::trunc);
//   out2 << cube22;
//   cout << "Read out a.cube, then write it as b.cube" << endl;
//   out2.close();

//   cout<<"Cube success: hello, world!" << endl << endl;
  
// }
