/****************************************************************************
**  Copyright (C) 2004                                                     **
**  University of Tennessee, Innovative Computing Laboratory               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/


#ifndef CUBE_THREAD_H
#define CUBE_THREAD_H   

/*
 *----------------------------------------------------------------------------
 *
 * class Thread
 *
 *----------------------------------------------------------------------------
 */

#include <iostream>
#include <vector>
#include <map>

#include "Location.h"

namespace cube {
  class Process;
  class Thread: public Location {
    
  public:
    Thread(std::string name, int rank, Process* proc): Location(name), rank(rank), proc(proc) {}

    const Process* get_proc()     const  { return proc; }
    int get_rank()                const  { return rank; }
    const Vertex*  get_child(int) const  { return NULL; } // already leaf node
    int   num_children()          const  { return 0; }    // already leaf node
      
    void writeXML(std::ostream& out, int& loc_id,
		  std::map<const Location*, int>& loc_mapping) const;

  private:
    Process* proc; // parent node
    int rank;
  };
  
  bool operator== (const Thread& a, const Thread& b);

}

#endif
