/****************************************************************************
**  Copyright (C) 2004                                                     **
**  University of Tennessee, Innovative Computing Laboratory               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/


#ifndef CUBE_VERTEX_H
#define CUBE_VERTEX_H   

/*
 *----------------------------------------------------------------------------
 *
 * class Vertex 
 *
 *----------------------------------------------------------------------------
 */

#include <string>
#include <functional>

#define INDENTSPACE 2

namespace cube {
  class Vertex {
    
  public:
    Vertex() {};
    
    void  set_level(int level) const { this->level = level; }
    int   get_level()    const { return level;  }
    std::string indent() const { return std::string(INDENTSPACE*level,' '); }
    
    virtual int   num_children()           const = 0;
    virtual const Vertex* get_child(int i) const = 0;
    
  private:
    mutable int level;    
  };

  
  //binary predicate
  template<class T>
    class GenLess : public std::binary_function<T, T, bool> {
    public :
      bool operator() (const T a, const T b) const {
      return (*a < *b);
    }
  };

}

#endif
