/****************************************************************************
**  KOJAK                                                                  **
**  http://www.fz-juelich.de/zam/kojak/                                    **
*****************************************************************************
**  Copyright (c) 1998-2004                                                **
**  Forschungszentrum Juelich, Zentralinstitut fuer Angewandte Mathematik  **
**                                                                         **
**  Copyright (c) 2003-2004                                                **
**  University of Tennessee, Innovative Computing Laboratory               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#ifndef EARL_EVENT_REP_H
#define EARL_EVENT_REP_H

/*
 *----------------------------------------------------------------------------
 *
 * class Event_rep
 *
 * This class represents the actual EARL events. Its an abstract class
 * containing the base attributes and defining a common interface to
 * events.
 *
 *----------------------------------------------------------------------------
 */

#include <string>

#include "Error.h"
#include "Event.h"

namespace earl 
{
  class Callsite;
  class Communicator;
  class EventBuffer;
  class Location;
  class Region;
  class State;

  class Event_rep
  {
  public:
  
    Event_rep(State&        state,
	      EventBuffer*  buffer,
	      Location*     loc, 
	      double        time);
  
    virtual ~Event_rep() {} 
  
    long add_ref();    
    long rem_ref(); 
  
    long         get_pos()      const { return pos; }
    Location*    get_loc()      const { return loc; }
    double       get_time()     const { return time; }
    Event        get_enterptr() const { return enterptr; }

    virtual bool           is_type(etype type) const; 

    virtual etype          get_type()         const { throw RuntimeError("No such type."); }
    virtual std::string    get_typestr()      const { throw RuntimeError("No such type."); }
    virtual Event          get_cnodeptr()     const { return NULL; }
    virtual Event          get_cedgeptr()     const { return NULL; }
    virtual Region*        get_reg()          const { return NULL; }
    virtual Callsite*      get_csite()        const { return NULL; }
    virtual Event          get_sendptr()      const { return NULL; }
    virtual Event          get_lockptr()      const { return NULL; }
    virtual Event          get_forkptr()      const { return NULL; }
    virtual Location*      get_src()          const { return NULL; }
    virtual Location*      get_dest()         const { return NULL; }
    virtual Communicator*  get_com()          const { return NULL; }
    virtual long           get_tag()          const { return -1; }
    virtual long           get_length()       const { return -1; }
    virtual Location*      get_root()         const { return NULL; }
    virtual long           get_sent()         const { return -1; }
    virtual long           get_recvd()        const { return -1; }
    virtual long           get_lock_id()      const { return -1; }

    virtual long           get_nmets()         const { return 0; }
    virtual long long      get_metval(long i)  const { throw RuntimeError("No such metric."); }
    virtual std::string    get_metname(long i) const { throw RuntimeError("No such metric.");}

    // should take S(pos-1) as argument
    virtual void trans(State& state);

    void detach_buffer() { buffer = NULL; }

  private:

    long         pos;
    Location*    loc;
    double       time;
  
    Event  enterptr;
  
    EventBuffer* buffer;
    long         refc;  
  };
}
#endif






