/****************************************************************************
**  KOJAK                                                                  **
**  http://www.fz-juelich.de/zam/kojak/                                    **
*****************************************************************************
**  Copyright (c) 1998-2004                                                **
**  Forschungszentrum Juelich, Zentralinstitut fuer Angewandte Mathematik  **
**                                                                         **
**  Copyright (c) 2003-2004                                                **
**  University of Tennessee, Innovative Computing Laboratory               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#include "MPI_CollectiveQueue.h"

#include <algorithm>

#include "Communicator.h"
#include "Grid.h"

using namespace earl;
using namespace std;

MPI_CollectiveQueue::MPI_CollectiveQueue(const Grid* grid) : 
  completed(NULL),
  mpicexitv(grid->get_ncoms())
{
  for ( int com_id = 0; com_id < mpicexitv.size(); com_id++ )
    for ( int rank = 0; rank < grid->get_com(com_id)->size(); rank++ )
      mpicexitv[com_id].push_back(list<Event>());
}

void   
MPI_CollectiveQueue::mpicoll(vector<Event>& out) 
{
  if ( completed )
    for (int rank = 0; rank < completed->size(); rank++)
      {
	long com_id = completed->get_id();
	
	out.push_back(mpicexitv[com_id][rank].front());
      }

  sort(out.begin(), out.end());
} 

void   
MPI_CollectiveQueue::queue_mpicoll(Event cexit)
{
  bool is_completed = true;
  
  Communicator* com = cexit.get_com();
  Process* proc     = cexit.get_loc()->get_proc();

  long com_id = com->get_id();
  long rank   = com->get_rank(cexit.get_loc());

  mpicexitv[com_id][rank].push_back(cexit);
	    
  for ( rank = 0; rank < com->size(); rank++ )
    if ( mpicexitv[com_id][rank].empty() )
      is_completed = false;
  
  if ( is_completed )
    completed = com;
}

void   
MPI_CollectiveQueue::dequeue_mpicoll()
{
  if ( completed )
    for (int rank = 0; rank < completed->size(); rank++)
      {
	long com_id = completed->get_id();
	
	mpicexitv[com_id][rank].pop_front() ;
      }
  completed = NULL;
}













