/****************************************************************************
**  KOJAK                                                                  **
**  http://www.fz-juelich.de/zam/kojak/                                    **
*****************************************************************************
**  Copyright (c) 1998-2004                                                **
**  Forschungszentrum Juelich, Zentralinstitut fuer Angewandte Mathematik  **
**                                                                         **
**  Copyright (c) 2003-2004                                                **
**  University of Tennessee, Innovative Computing Laboratory               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#include "OMP_CollectiveQueue.h"

#include <algorithm>

#include "Grid.h"
#include "Location.h"
#include "Process.h"
#include "Thread.h"

using namespace earl;
using namespace std;


OMP_CollectiveQueue::OMP_CollectiveQueue(const Grid* grid) : 
  completed(NULL),
  ompcexitv(grid->get_nprocs())
{
  for ( int proc_id = 0; proc_id < ompcexitv.size(); proc_id++ )
    for ( int thrd_id = 0; thrd_id < grid->get_proc(proc_id)->get_nthrds(); thrd_id++ )
      ompcexitv[proc_id].push_back(list<Event>());
}


void   
OMP_CollectiveQueue::ompcoll(vector<Event>& out) 
{
  if ( completed )
    for (int thrd_id = 0; thrd_id < completed->get_nthrds(); thrd_id++)
      {
	long proc_id = completed->get_id();
	
	out.push_back(ompcexitv[proc_id][thrd_id].front());
      }
  sort(out.begin(), out.end());
}

void   
OMP_CollectiveQueue::queue_ompcoll(Event cexit)
{
  bool is_completed = true;
  
  Process* proc = cexit.get_loc()->get_proc();
  Thread*  thrd = cexit.get_loc()->get_thrd();
  long proc_id = proc->get_id();

  ompcexitv[proc_id][thrd->get_thrd_id()].push_back(cexit);
	    
  for ( int thrd_id = 0; thrd_id < proc->get_nthrds(); thrd_id++ )
    if ( ompcexitv[proc_id][thrd_id].empty() )
      is_completed = false;
  
  if ( is_completed )
    completed = proc;
}

void   
OMP_CollectiveQueue::dequeue_ompcoll()
{
  if ( completed )
    for (int thrd_id = 0; thrd_id < completed->get_nthrds(); thrd_id++)
      {
	long proc_id = completed->get_id();
	
	ompcexitv[proc_id][thrd_id].pop_front();
      }
  completed = NULL;  
}







