%typemap(python, in) long 
{
  if (PyInt_Check($input))
    {
      $1 = PyInt_AsLong($input);
    }
  else if (PyLong_Check($input))
    {
      $1 = PyLong_AsLong($input);
    }
  else
    {
      PyErr_SetString(PyExc_TypeError, "not an integer");
      return NULL;
    }
}

%typemap(python, in) std::string 
{

  if (PyString_Check($input))
    {
      $1 = PyString_AsString($input);
    }
  else
    {
      PyErr_SetString(PyExc_TypeError, "not a string");
      return NULL;
    }
}

%typemap(python, in) longv_in& (longv_in lv_in)
{
  if(PyList_Check($input))
    {
      $1 = &lv_in;

      for ( int i = 0 ; i < PyList_Size($input) ; i++ ) 
        {
          ($1)->push_back(PyInt_AsLong(PyList_GetItem($input, i)));
        }
    }
  else
    {
      PyErr_SetString(PyExc_TypeError, "not a list");
      return NULL;
    }
}


%typemap(python, in) etype
{
  if (PyString_Check($input))
    {
      if ( strcmp(PyString_AsString($input), "ALOCK") == 0 )
	$1 = ALOCK;
      else if ( strcmp(PyString_AsString($input), "ENTER") == 0 )
	$1 = ENTER;
      else if ( strcmp(PyString_AsString($input), "EVENT") == 0 )
	$1 = EVENT;
      else if ( strcmp(PyString_AsString($input), "EXIT") == 0 )
	$1 = EXIT;
      else if ( strcmp(PyString_AsString($input), "FLOW") == 0 )
	$1 = FLOW;
      else if ( strcmp(PyString_AsString($input), "FORK") == 0 )
	$1 = FORK;
      else if ( strcmp(PyString_AsString($input), "JOIN") == 0 )
	$1 = JOIN;
      else if ( strcmp(PyString_AsString($input), "MPICEXIT") == 0 )
	$1 = MPICEXIT;
      else if ( strcmp(PyString_AsString($input), "OMPCEXIT") == 0 )
	$1 = OMPCEXIT;
      else if ( strcmp(PyString_AsString($input), "P2P") == 0 )
	$1 = P2P;
      else if ( strcmp(PyString_AsString($input), "RECV") == 0 )
	$1 = RECV;
      else if ( strcmp(PyString_AsString($input), "RLOCK") == 0 )
	$1 = RLOCK;
      else if ( strcmp(PyString_AsString($input), "SEND") == 0 )
	$1 = SEND;
      else if ( strcmp(PyString_AsString($input), "SYNC") == 0 )
	$1 = SYNC;
      else if ( strcmp(PyString_AsString($input), "TEAM") == 0 )
	$1 = TEAM;
      else 
	{
	  PyErr_SetString(PyExc_TypeError, "no such type");
	  return NULL;
	}
    }
  else
    {
      PyErr_SetString(PyExc_TypeError, "not a string");
      return NULL;
    }
}

%typemap(python, out) std::string 
{
  $result = PyString_FromString(const_cast<char *>($1.c_str()));
}

%typemap(python, out) Location*
{
  PyObject* item;

  $result = PyDict_New();

  item = PyInt_FromLong($1->get_id());  
  PyDict_SetItemString($result, "id", item);
  Py_DECREF(item);
  item = PyInt_FromLong($1->get_mach()->get_id());  
  PyDict_SetItemString($result, "mach_id", item);
  Py_DECREF(item);
  item = PyInt_FromLong($1->get_node()->get_node_id());  
  PyDict_SetItemString($result, "node_id", item);
  Py_DECREF(item);
  item = PyInt_FromLong($1->get_proc()->get_id());  
  PyDict_SetItemString($result, "proc_id", item);
  Py_DECREF(item);
  item = PyInt_FromLong($1->get_thrd()->get_thrd_id());  
  PyDict_SetItemString($result, "thrd_id", item);
  Py_DECREF(item);
}

%typemap(python, out) Machine*
{
  PyObject* item;

  $result = PyDict_New();

  item = PyInt_FromLong($1->get_id());  
  PyDict_SetItemString($result, "id", item);
  Py_DECREF(item);
  item = PyString_FromString(const_cast<char*>($1->get_name().c_str()));   
  PyDict_SetItemString($result, "name", item);
  Py_DECREF(item);
  item = PyInt_FromLong($1->get_nnodes());  
  PyDict_SetItemString($result, "nnodes", item);
  Py_DECREF(item);
}

%typemap(python, out) Node*
{
  PyObject* item;

  $result = PyDict_New();

  item = PyInt_FromLong($1->get_mach_id());  
  PyDict_SetItemString($result, "mach_id", item);
  Py_DECREF(item);
  item = PyInt_FromLong($1->get_node_id());  
  PyDict_SetItemString($result, "node_id", item);
  Py_DECREF(item);
  item = PyString_FromString(const_cast<char*>($1->get_name().c_str()));   
  PyDict_SetItemString($result, "name", item);
  Py_DECREF(item);
  item = PyInt_FromLong($1->get_ncpus());  
  PyDict_SetItemString($result, "ncpus", item);
  Py_DECREF(item);
  item = PyFloat_FromDouble($1->get_clckrt());  
  PyDict_SetItemString($result, "clckrt", item);
  Py_DECREF(item);
  item = PyInt_FromLong($1->get_nprocs());  
  PyDict_SetItemString($result, "nprocs", item);
  Py_DECREF(item);
}

%typemap(python, out) Process*
{
  PyObject* item;

  $result = PyDict_New();

  item = PyInt_FromLong($1->get_id());  
  PyDict_SetItemString($result, "id", item);
  Py_DECREF(item);
  item = PyString_FromString(const_cast<char*>($1->get_name().c_str()));   
  PyDict_SetItemString($result, "name", item);
  Py_DECREF(item);
  item = PyInt_FromLong($1->get_nthrds());  
  PyDict_SetItemString($result, "nthrds", item);
  Py_DECREF(item);
  item = PyInt_FromLong($1->get_loc()->get_id());  
  PyDict_SetItemString($result, "loc_id", item);
  Py_DECREF(item);
  item = PyInt_FromLong($1->get_node()->get_node_id());  
  PyDict_SetItemString($result, "node_id", item);
  Py_DECREF(item);
  item = PyInt_FromLong($1->get_node()->get_mach_id());  
  PyDict_SetItemString($result, "mach_id", item);
  Py_DECREF(item);
}

%typemap(python, out) Thread*
{
  PyObject* item;

  $result = PyDict_New();

  item = PyInt_FromLong($1->get_proc_id());  
  PyDict_SetItemString($result, "proc_id", item);
  Py_DECREF(item);
  item = PyInt_FromLong($1->get_thrd_id());  
  PyDict_SetItemString($result, "thrd_id", item);
  Py_DECREF(item);
  item = PyString_FromString(const_cast<char*>($1->get_name().c_str()));   
  PyDict_SetItemString($result, "name", item);
  Py_DECREF(item);
  item = PyInt_FromLong($1->get_loc()->get_id());  
  PyDict_SetItemString($result, "loc_id", item);
  Py_DECREF(item);
}

%typemap(python, out) Callsite*
{
  PyObject* item;

  $result = PyDict_New();

  item = PyInt_FromLong($1->get_id());  
  PyDict_SetItemString($result, "id", item);
  Py_DECREF(item);
  item = PyString_FromString(const_cast<char*>($1->get_file().c_str()));   
  PyDict_SetItemString($result, "file", item);
  Py_DECREF(item);
  item = PyInt_FromLong($1->get_line());  
  PyDict_SetItemString($result, "line", item);
  Py_DECREF(item);
  item = PyInt_FromLong($1->get_callee()->get_id());   
  PyDict_SetItemString($result, "callee_id", item);
  Py_DECREF(item);
}


%typemap(python, out) Region*
{
  PyObject* item;

  $result = PyDict_New();

  item = PyInt_FromLong($1->get_id());  
  PyDict_SetItemString($result, "id", item);
  Py_DECREF(item);
  item = PyString_FromString(const_cast<char*>($1->get_name().c_str()));   
  PyDict_SetItemString($result, "name", item);
  Py_DECREF(item);
  item = PyString_FromString(const_cast<char*>($1->get_file().c_str()));   
  PyDict_SetItemString($result, "file", item);
  Py_DECREF(item);
  item = PyInt_FromLong($1->get_begln());  
  PyDict_SetItemString($result, "begln", item);
  Py_DECREF(item);
  item = PyInt_FromLong($1->get_endln());  
  PyDict_SetItemString($result, "endln", item);
  Py_DECREF(item);
  item = PyString_FromString(const_cast<char*>($1->get_rtype().c_str()));   
  PyDict_SetItemString($result, "rtype", item);
  Py_DECREF(item);
  item = PyString_FromString(const_cast<char*>($1->get_descr().c_str()));   
  PyDict_SetItemString($result, "descr", item);
  Py_DECREF(item);
}

%typemap(python, out) Metric*
{
  PyObject* item;

  $result = PyDict_New();

  item = PyInt_FromLong($1->get_id());  
  PyDict_SetItemString($result, "id", item);
  Py_DECREF(item);
  item = PyString_FromString(const_cast<char*>($1->get_name().c_str()));   
  PyDict_SetItemString($result, "name", item);
  Py_DECREF(item);
  item = PyString_FromString(const_cast<char*>($1->get_descr().c_str()));   
  PyDict_SetItemString($result, "descr", item);
  Py_DECREF(item);
  item = PyString_FromString(const_cast<char*>($1->get_type().c_str()));   
  PyDict_SetItemString($result, "type", item);
  Py_DECREF(item);
  item = PyString_FromString(const_cast<char*>($1->get_mode().c_str()));   
  PyDict_SetItemString($result, "mode", item);
  Py_DECREF(item);
  item = PyString_FromString(const_cast<char*>($1->get_ival().c_str()));   
  PyDict_SetItemString($result, "ival", item);
  Py_DECREF(item);
}

%typemap(python, out) Communicator*
{ 
  $result = PyList_New($1->size());
  
  for ( int i = 0; i < $1->size(); i++)
    {
      PyObject* item = PyInt_FromLong($1->get_proc(i)->get_id());
      PyList_SetItem($result, i, item);
    }
}

%typemap(python, out) Cartesian*
{
  PyObject* item;	

  $result = PyDict_New();

  item = PyInt_FromLong($1->get_id());  
  PyDict_SetItemString($result, "id", item);
  Py_DECREF(item);
  item = PyInt_FromLong($1->get_ndims());  
  PyDict_SetItemString($result, "ndims", item);
  Py_DECREF(item);

  std::vector<long> dimv;	
  $1->get_dimv(dimv);   
  item = PyList_New($1->get_ndims());
  for (int i = 0 ; i < $1->get_ndims() ; i++)
    {
      PyObject* item2;	
      item2 = PyInt_FromLong(dimv[i]);
      PyList_SetItem(item, i, item2);    	 	
    }  	
  PyDict_SetItemString($result, "dimv", item);
  Py_DECREF(item);	


  std::vector<bool> periodv;	
  $1->get_periodv(periodv);   
  item = PyList_New($1->get_ndims());
  for (int i = 0 ; i < $1->get_ndims() ; i++)
    {
      PyObject* item2;	
      item2 = PyInt_FromLong(periodv[i]);
      PyList_SetItem(item, i, item2);    	 	
    }  	
  PyDict_SetItemString($result, "periodv", item);
  Py_DECREF(item);

}

%typemap(python, out) Event
{ 
  PyObject* item;

  $result = PyDict_New();

  item = PyInt_FromLong($1.get_pos());  
  PyDict_SetItemString($result, "pos", item);
  Py_DECREF(item);
  item = PyInt_FromLong($1.get_loc()->get_id());  
  PyDict_SetItemString($result, "loc_id", item);
  Py_DECREF(item);
  item = PyFloat_FromDouble($1.get_time());  
  PyDict_SetItemString($result, "time", item);
  Py_DECREF(item);
  item = PyString_FromString(const_cast<char*>($1.get_typestr().c_str()));   
  PyDict_SetItemString($result, "type", item);
  Py_DECREF(item);
  if ( ! $1.get_enterptr().null() )
    item = PyInt_FromLong($1.get_enterptr().get_pos());  
  else
    item = PyInt_FromLong(-1);
  PyDict_SetItemString($result, "enterptr", item);
  Py_DECREF(item);

  if ( $1.is_type(FLOW) )
    {
      item = PyInt_FromLong($1.get_reg()->get_id());  
      PyDict_SetItemString($result, "reg_id", item);
      Py_DECREF(item);

      for ( int i = 0; i < $1.get_nmets(); i++ )	
	{
	  item = PyLong_FromLongLong($1.get_metval(i));  
	  PyDict_SetItemString($result, const_cast<char*>($1.get_metname(i).c_str()), item);
	  Py_DECREF(item);
	}
    }
  if ( $1.is_type(ENTER) )
    {
      item = PyInt_FromLong($1.get_cnodeptr().get_pos());  
      PyDict_SetItemString($result, "cnodeptr", item);
      Py_DECREF(item);
      if ( ! $1.get_cedgeptr().null() )
	item = PyInt_FromLong($1.get_cedgeptr().get_pos());  
      else
	item = PyInt_FromLong(-1);
      PyDict_SetItemString($result, "cedgeptr", item);
      Py_DECREF(item);

      if ( $1.get_csite() )
	item = PyInt_FromLong($1.get_csite()->get_id() );  
      else
	item = PyInt_FromLong(-1);
      PyDict_SetItemString($result, "csite_id", item);
      Py_DECREF(item);
    }
  if ( $1.is_type(MPICEXIT) )
    {
      if ( $1.get_root() != NULL )
	item = PyInt_FromLong($1.get_root()->get_id());
      else
	item = PyInt_FromLong(-1);	
      PyDict_SetItemString($result, "root_id", item);
      Py_DECREF(item);
      item = PyInt_FromLong($1.get_com()->get_id());  
      PyDict_SetItemString($result, "com_id", item);
      Py_DECREF(item);
      item = PyInt_FromLong($1.get_sent());  
      PyDict_SetItemString($result, "sent", item);
      Py_DECREF(item);
      item = PyInt_FromLong($1.get_recvd());  
      PyDict_SetItemString($result, "recvd", item);
      Py_DECREF(item);
    }
  if ( $1.is_type(P2P) )
    {
      item = PyInt_FromLong($1.get_com()->get_id());  
      PyDict_SetItemString($result, "com_id", item);
      Py_DECREF(item);
      item = PyInt_FromLong($1.get_tag());  
      PyDict_SetItemString($result, "tag", item);
      Py_DECREF(item);
      item = PyInt_FromLong($1.get_length());  
      PyDict_SetItemString($result, "length", item);
      Py_DECREF(item);
    }
  if ( $1.is_type(SEND) )
    {
      item = PyInt_FromLong($1.get_dest()->get_id());  
      PyDict_SetItemString($result, "dest_id", item);
      Py_DECREF(item);
    }     
  if ( $1.is_type(RECV) )
    {
      item = PyInt_FromLong($1.get_src()->get_id());  
      PyDict_SetItemString($result, "src_id", item);
      Py_DECREF(item);
      item = PyInt_FromLong($1.get_sendptr().get_pos());  
      PyDict_SetItemString($result, "sendptr", item);
      Py_DECREF(item);
    }     
  if ( $1.is_type(SYNC) )
    {
      item = PyInt_FromLong($1.get_lock_id());  
      PyDict_SetItemString($result, "lock_id", item);
      Py_DECREF(item);
      if ( ! $1.get_lockptr().null() )
	item = PyInt_FromLong($1.get_lockptr().get_pos());  
      else
	item = PyInt_FromLong(-1);
      PyDict_SetItemString($result, "lockptr", item);
      Py_DECREF(item);
    }
  if ( $1.is_type(JOIN) )
    {
      item = PyInt_FromLong($1.get_forkptr().get_pos());  
      PyDict_SetItemString($result, "forkptr", item);
      Py_DECREF(item);
    }
}

%typemap(python, in,numinputs=0) std::vector<Event>& (std::vector<Event> ev)
{
  $1 = &ev;
}

%typemap(python, argout) std::vector<Event>&
{ 
  $result = PyList_New($1->size());
  
  for ( int i = 0; i < $1->size(); i++)
    {
      PyObject* item = PyInt_FromLong((*$1)[i].get_pos());
      PyList_SetItem($result, i, item);
    }
}

%typemap(python, in,numinputs=0) longv_out& (longv_out lv_out)
{
  $1 = &lv_out;
}

%typemap(python, argout) longv_out&
{ 
  $result = PyList_New($1->size());
  
  for ( int i = 0; i < $1->size(); i++)
    {
      PyObject* item = PyInt_FromLong((*$1)[i]);
      PyList_SetItem($result, i, item);
    }
}

%exception
{	
  try 	
    {	
      $action
    }	
  catch(RuntimeError e) 
    {
      PyErr_SetString(PyExc_RuntimeError, const_cast<char*>(e.get_msg().c_str()));   
      return NULL;
    }
  catch(FatalError e) 
    {
      PyErr_SetString(PyExc_SystemExit, const_cast<char*>(e.get_msg().c_str()));   
      return NULL;
    }
}













