/****************************************************************************
**  KOJAK                                                                  **
**  http://www.fz-juelich.de/zam/kojak/                                    **
*****************************************************************************
**  Copyright (c) 2003-2004                                                **
**  University of Tennessee                                                **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#ifndef _EXECUTABLE_H
#define _EXECUTABLE_H   

/*
 *----------------------------------------------------------------------------
 *
 * class Executable
 *
 *----------------------------------------------------------------------------
 */

#include <iostream>
  using std::ostream;
  using std::endl;

#include <map>
  using std::map;

#include <string>
  using std::string;

#include <vector>
  using std::vector;

#include <dpcl.h>

#include "Module.h"
#include "Function.h"
#include "Callsite.h"

class Executable
{
 public:
  
  // constructor and destructor
 
  Executable(Process proc, string prog_name);
  ~Executable();

  string get_name() const { return name; }

  // program structure

  int get_num_mods()   const { return modv.size(); }
  int get_num_funcs()  const { return funcv.size(); }
  int get_num_csites() const { return csitev.size(); }

  Module*   get_mod(int i)   const { return modv[i]; }
  Function* get_func(int i)  const { return funcv[i]; } 
  Callsite* get_csite(int i) const { return csitev[i]; } 
  
  Module*   get_mod(string name) const;
  Function* get_func(string name) const;

 private:

  string name;

  map<string, Module*>   modm;
  map<string, Function*> funcm;
  vector<Module*>        modv;
  vector<Function*>      funcv;
  vector<Callsite*>      csitev;
  
  Process proc;
};

ostream& operator<<(ostream& out, const Executable& mod); 

#endif














