/****************************************************************************
**  KOJAK                                                                  **
**  http://www.fz-juelich.de/zam/kojak/                                    **
*****************************************************************************
**  Copyright (c) 2003-2004                                                **
**  University of Tennessee                                                **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#include "Function.h"

#include <cctype>

#include "Callsite.h"
#include "Module.h"

int cmp_nocase(const string& lhs, const string& rhs)
{
  string::const_iterator li= lhs.begin();
  string::const_iterator ri= rhs.begin();

  while ( li != lhs.end() && ri != rhs.end() )
    {
      if ( toupper(*li) != toupper(*ri) )
	return ( toupper(*li) < toupper(*ri) ) ? -1 : 1;
      ++li;
      ++ri;
    }
  return ( lhs.size() == rhs.size() ) ? 0 : ( lhs.size() < rhs.size() ) ? -1 : 1;
}

bool
Function::is_mpi()
{
  if ( this->name.compare(0, 3, "MPI") == 0 || this->name.compare(0, 3, "mpi") == 0 )
    return true;
  else
    return false;
}

bool
Function::is_mpi_coll()
{
  static char* collv[] = { "mpi_allgather",
			   "mpi_allgatherv",		
			   "mpi_allreduce",		
			   "mpi_alltoall",		
			   "mpi_alltoallv",
			   "mpi_barrier",
			   "mpi_bcast",               
			   "mpi_gather",              
			   "mpi_gatherv",             
			   "mpi_op_create",		
			   "mpi_op_free",		
			   "mpi_reduce",			
			   "mpi_reduce_scatter",	
			   "mpi_scan",		
			   "mpi_scatter",             
			   "mpi_scatterv",
                           NULL };
  char** it = collv;
  while ( *it != NULL )
    {
      if ( cmp_nocase(*it, this->name) == 0 )
	return true;
      it++;
    }
  return false;
}

bool
Function::is_omp()
{
  if ( this->name.compare(0, 3, "OMP") == 0 || this->name.compare(0, 3, "omp") == 0 )
    return true;
  else
    return false;
}

ostream& operator<<(ostream& out, const Function& func)
{
  out << "FUNCTION" << " "
      << func.id << " " 
      << func.name << " "
      << func.begln << " "
      << func.endln << " "
      << func.mod->id << " "
      << func.csitev.size();
  
  for (int i = 0; i < func.csitev.size(); i++)
    out << " " << func.csitev[i]->id;

  out << endl;

  return out;
}
