!****************************************************************************
!*  KOJAK                                                                  **
!*  http://www.fz-juelich.de/zam/kojak/                                    **
!****************************************************************************
!*  Copyright (c) 1998-2004                                                **
!*  Forschungszentrum Juelich, Zentralinstitut fuer Angewandte Mathematik  **
!*                                                                         **
!*  See the file COPYRIGHT in the package base directory for details       **
!***************************************************************************/

subroutine step(id, tid, i)
  integer :: id, tid, i

  !$POMP INST BEGIN(step)
  write (*,*) id, ":", tid, ": step", i
  !$POMP INST END(step)
end subroutine

subroutine sequential(id)
  integer :: i, tid=0
#if (defined (ELG_OMPI) || defined (ELG_OMP))
  integer :: omp_get_thread_num
#endif

  !$POMP INST BEGIN(sequential)

  !$OMP PARALLEL PRIVATE(tid)
#if (defined (ELG_OMPI) || defined (ELG_OMP))
    tid = omp_get_thread_num()
#endif
    !$OMP DO
    do i=1,12
      call step(id, tid, i)
    enddo
    !$OMP END DO
  !$OMP END PARALLEL

  !$POMP INST END(sequential)
end subroutine

program main
#if (defined (ELG_MPI) || defined (ELG_OMPI))
  include 'mpif.h'
#endif
  integer :: id=0, numprocs, value=42, ierr

  !$POMP INST INIT
  !$POMP INST BEGIN(main)
  
#if (defined (ELG_MPI) || defined (ELG_OMPI))
  call MPI_Init(ierr)
  call MPI_Comm_size(MPI_COMM_WORLD, numprocs, ierr)
  call MPI_Comm_rank(MPI_COMM_WORLD, id, ierr)
  call MPI_Bcast(value, 1, MPI_INTEGER, numprocs-1, MPI_COMM_WORLD, ierr)
#endif

  call sequential(id)

#if (defined (ELG_MPI) || defined (ELG_OMPI))
  call MPI_Barrier(MPI_COMM_WORLD, ierr)
  call MPI_Finalize(ierr)
#endif

  !$POMP INST END(main)
end program main
