/****************************************************************************
**  KOJAK                                                                  **
**  http://www.fz-juelich.de/zam/kojak/                                    **
*****************************************************************************
**  Copyright (c) 1998-2004                                                **
**  Forschungszentrum Juelich, Zentralinstitut fuer Angewandte Mathematik  **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "elg_user.h"

#if (defined (ELG_OMPI) || defined (ELG_OMP))
  #include "opari_omp.h"
#endif

#if (defined (ELG_MPI) || defined (ELG_OMPI))
#include <mpi.h>
#endif

void step(int id, int tid, int i) {  
  #ifdef generic
  ELG_USER_START("step");
  #endif
  #ifdef pomp
  #pragma pomp inst begin(step)
  #endif

  printf("%d:%d: step %d\n", id, tid, i);

  #ifdef pomp
  #pragma pomp inst end(step)
  #endif
  #ifdef generic
  ELG_USER_END("step");
  #endif
}

void sequential(int id) {
  int i, tid=0;

  #ifdef generic
  ELG_USER_START("sequential");
  #endif
  #ifdef pomp
  #pragma pomp inst begin(sequential)
  #endif

  sleep(1);
  #pragma omp parallel private(tid)
  {
    #if (defined (ELG_OMPI) || defined (ELG_OMP))
    tid = omp_get_thread_num();
    #endif
    #pragma omp for
    for (i=0; i<12; i++) {
      step(id, tid, i);
    }    
  }
  sleep(1);

  #ifdef pomp
  #pragma pomp inst end(sequential)
  #endif
  #ifdef generic
  ELG_USER_END("sequential");
  #endif
}

int main(int argc, char* argv[]) {
  int id=0, numprocs, value=42;

  #ifdef generic
  ELG_USER_START("main");
  #endif
  #ifdef pomp
  #pragma pomp inst init
  #pragma pomp inst begin(main)
  #endif
  
  #if (defined (ELG_MPI) || defined (ELG_OMPI))
  MPI_Init(&argc, &argv);
  MPI_Comm_size(MPI_COMM_WORLD, &numprocs);
  MPI_Comm_rank(MPI_COMM_WORLD, &id);
  MPI_Bcast(&value, 1, MPI_INT, numprocs-1, MPI_COMM_WORLD);
  #endif

  sequential(id);

  #if (defined (ELG_MPI) || defined (ELG_OMPI))
  MPI_Barrier(MPI_COMM_WORLD);
  MPI_Finalize();
  #endif

  #ifdef pomp
  #pragma pomp inst end(main)
  #endif
  #ifdef generic
  ELG_USER_END("main");
  #endif
  return 0;
}
