/****************************************************************************
**  KOJAK                                                                  **
**  http://www.fz-juelich.de/zam/kojak/                                    **
*****************************************************************************
**  Copyright (c) 1998-2004                                                **
**  Forschungszentrum Juelich, Zentralinstitut fuer Angewandte Mathematik  **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "elg_trc.h"
#include "elg_user.h"

#if (defined (ELG_OMPI) || defined (ELG_OMP))
  #include "opari_omp.h" 
#endif

#if (defined (ELG_MPI) || defined (ELG_OMPI))
#include <mpi.h>
#endif

#if (defined (ELG_MPI) || defined (ELG_OMPI))
int nbr(MPI_Comm comm)
{
  int me, numprocs;
  MPI_Comm_rank(comm, &me);
  MPI_Comm_size(comm, &numprocs);

  return (me + 1) % numprocs;
}

int nbl(MPI_Comm comm)
{
  int me, numprocs;
  MPI_Comm_rank(comm, &me);
  MPI_Comm_size(comm, &numprocs);

  return (me + numprocs - 1) % numprocs;
}

void print_msg(char *msg)
{
  int me;
  MPI_Comm_rank(MPI_COMM_WORLD, &me);
  MPI_Barrier(MPI_COMM_WORLD);
  if (me == 0)
    printf("%s", msg);
}

void comm_test(MPI_Comm comm)
{
  MPI_Comm newcomm;
  int me, numprocs, msg = 1;
  MPI_Status stat;

  MPI_Comm_rank(comm, &me);
  MPI_Comm_size(comm, &numprocs);

  if (numprocs > 1) 
    {
      MPI_Comm_split(comm, (me % 2), me, &newcomm);

      /*- action within the new communicator -*/
      MPI_Barrier(newcomm);
      MPI_Send(&msg, 1, MPI_INT, nbr(comm), 1, comm);  
      MPI_Recv(&msg, 1, MPI_INT, nbl(comm), 1, comm, &stat);

      comm_test(newcomm);
      MPI_Comm_free(&newcomm);
    }
}

void req_test(MPI_Comm comm)
{
  int i;
  int me, numprocs, msg = 1;
  MPI_Status stat;
  MPI_Request req[10];

  MPI_Comm_rank(comm, &me);
  MPI_Comm_size(comm, &numprocs);
  
  for (i = 0; i < 5; i++)
    {
      MPI_Isend(&msg, 1, MPI_INT, nbr(comm), i, comm, &req[2*i]);
      MPI_Irecv(&msg, 1, MPI_INT, nbl(comm), i, comm, &req[2*i+1]);
    }
  for (i = 0; i < 10; i++)
      MPI_Wait(&req[i], &stat);
}

void pers_req_test(MPI_Comm comm)
{
  int i;
  int me, numprocs, msg = 1;
  MPI_Status stat;
  MPI_Request req[10];

  MPI_Comm_rank(comm, &me);
  MPI_Comm_size(comm, &numprocs);
  
  for (i = 0; i < 5; i++)
    {
      MPI_Send_init(&msg, 1, MPI_INT, nbr(comm), i, comm, &req[2*i]);  
      MPI_Recv_init(&msg, 1, MPI_INT, nbl(comm), i, comm, &req[2*i+1]);

      MPI_Start(&req[2*i]);
      MPI_Start(&req[2*i+1]);
    }
  for (i = 0; i < 10; i++)
    {
      MPI_Wait(&req[i], &stat);
      MPI_Request_free(&req[i]);
    }
}

void ping_pong_test(int times, int delay, int msgsize, MPI_Comm comm)
{
  int i;
  int me, numprocs;
  MPI_Status stat;

  int nl = nbl(comm);
  int nr = nbr(comm);

  int* buf = malloc(msgsize * sizeof(int));

  MPI_Comm_rank(MPI_COMM_WORLD, &me);
  MPI_Comm_size(MPI_COMM_WORLD, &numprocs);
  
  for (i = 0; i < times; i++)
    {
      MPI_Barrier(comm);

      if (me % 2 == 0)
	{
	  MPI_Send(buf, msgsize, MPI_INT, nr, 2*numprocs*i+me, comm);  
	  MPI_Recv(buf, msgsize, MPI_INT, nl, 2*numprocs*(i+1)+nl, comm, &stat);
	  MPI_Send(buf, msgsize, MPI_INT, nl, 2*numprocs*i+me, comm);  
	  MPI_Recv(buf, msgsize, MPI_INT, nr, 2*numprocs*(i+1)+nr, comm, &stat);
	}
      else 
	{
	  MPI_Recv(buf, msgsize, MPI_INT, nl, 2*numprocs*i+nl, comm, &stat);
	  MPI_Send(buf, msgsize, MPI_INT, nr, 2*numprocs*(i+1)+me, comm);  
	  MPI_Recv(buf, msgsize, MPI_INT, nr, 2*numprocs*i+nr, comm, &stat);
	  MPI_Send(buf, msgsize, MPI_INT, nl, 2*numprocs*(i+1)+me, comm);  
	}
      if (i < times - 1) sleep(delay);
      if (me == 0) printf("%d\n", i);
    }

  free(buf);
}
#endif

void rec_call_test()
{
  static int i = 151;

  while(i)
    {
      i--;
      rec_call_test();
    }
}

#if (defined (ELG_OMPI) || defined (ELG_OMP))
void do_inside()
{
  int id = omp_get_thread_num();
    
  printf("I do work inside %d.\n", id);

#pragma omp barrier

  printf("I passed a barrier %d.\n", id);

#pragma omp critical
  {
    printf("I start critical work %d.\n", id);
    sleep(1); 
    printf("I finished critical work %d.\n", id);
  }

#pragma omp flush
}

void test_omp()
{
  omp_lock_t lck;
  int id;

  omp_init_lock(&lck);

#pragma omp parallel
  {    
    id = omp_get_thread_num();
    
    omp_set_lock(&lck);
    printf("My thread id is %d.\n", id);
    omp_unset_lock(&lck);
    
    while (! omp_test_lock(&lck))
      {
	printf("%d: skip().\n", id);
	sleep(1); 
      }
    
    printf("%d: work().\n", id);
    sleep(1); 
    
    omp_unset_lock(&lck);
    
    do_inside();
  }
  omp_destroy_lock(&lck);
}
#endif

int main(int argc, char** argv)

{
  int me = 0, numprocs;
#ifdef generic
  ELG_USER_START("main");
#endif

#if (defined (ELG_MPI) || defined (ELG_OMPI))
  MPI_Init(&argc, &argv);

  MPI_Comm_rank(MPI_COMM_WORLD, &me);
  MPI_Comm_size(MPI_COMM_WORLD, &numprocs);
  
  comm_test(MPI_COMM_WORLD);
  print_msg("Communicator test completed.\n");

  req_test(MPI_COMM_WORLD);
  print_msg("Request test completed.\n");

  pers_req_test(MPI_COMM_WORLD);
  print_msg("Persistent request test completed.\n");

  ping_pong_test(200, 0, 10000, MPI_COMM_WORLD);
  print_msg("Ping pong test completed.\n");

/*    rec_call_test();  */
/*    print_msg("Recursive call test completed.\n"); */
#endif

#if (defined (ELG_OMPI) || defined (ELG_OMP))
  test_omp();
#endif

#if (defined (ELG_MPI) || defined (ELG_OMPI))
  MPI_Finalize();
#endif

  printf("%d : Test completed.\n", me);
#ifdef generic
  ELG_USER_END("main");
#endif
  return 0;
}




































