#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "elg_user.h"
#include "elg_topol.h"

#if (defined (ELG_OMPI) || defined (ELG_OMP))
#include "opari_omp.h"
#endif

#if (defined (ELG_MPI) || defined (ELG_OMPI))
#include <mpi.h>
#endif

#define TRUE  1
#define FALSE 0

void mpi_cart_topol(int rank, int size)
{
  int dim[3], period[3], coord[3];

   /*  run this program with 8 processes */
  if ( size != 8 ) 
    {
      fprintf(stderr, "Invalid number of processes\n");
      exit(0);
    }
  
  dim[0] = 2;
  dim[1] = 2;
  dim[2] = 2;

  period[0] = TRUE;
  period[1] = FALSE;
  period[2] = TRUE;


  /* set the user topology using elg_topol.c routine */
  elg_cart_create(dim[0], dim[1], dim[2] , period[0], period[1], period[2]);

 
  if ( rank == 0 )
    {
      coord[0] = 0;
      coord[1] = 0;
      coord[2] = 0; 
      printf("P:%d My coordinates are %d %d %d\n", rank, coord[0], coord[1], coord[2]);
      elg_cart_coords(coord[0], coord[1], coord[2]);
    }
  else if ( rank == 1 ) 
    {
      coord[0] = 1;
      coord[1] = 0;
      coord[2] = 0; 
      printf("P:%d My coordinates are %d %d %d\n", rank, coord[0], coord[1], coord[2]);
      elg_cart_coords(coord[0], coord[1], coord[2]);
    }
  else if ( rank == 2 ) 
    {
      coord[0] = 0;
      coord[1] = 1;
      coord[2] = 0; 
      printf("P:%d My coordinates are %d %d %d\n", rank, coord[0], coord[1], coord[2]);
      elg_cart_coords(coord[0], coord[1], coord[2]);
    }
  else if ( rank == 3 ) 
    {
      coord[0] = 1;
      coord[1] = 1;
      coord[2] = 0; 
      printf("P:%d My coordinates are %d %d %d\n", rank, coord[0], coord[1], coord[2]);
      elg_cart_coords(coord[0], coord[1], coord[2]);
    }
  else if ( rank == 4 ) 
    {
      coord[0] = 0;
      coord[1] = 0;
      coord[2] = 1; 
      printf("P:%d My coordinates are %d %d %d\n", rank, coord[0], coord[1], coord[2]);
      elg_cart_coords(coord[0], coord[1], coord[2]);
    }
  else if ( rank == 5 ) 
    {
      coord[0] = 1;
      coord[1] = 0;
      coord[2] = 1; 
      printf("P:%d My coordinates are %d %d %d\n", rank, coord[0], coord[1], coord[2]);
      elg_cart_coords(coord[0], coord[1], coord[2]);
    }
  else if ( rank == 6 ) 
    {
      coord[0] = 0;
      coord[1] = 1;
      coord[2] = 1; 
      printf("P:%d My coordinates are %d %d %d\n", rank, coord[0], coord[1], coord[2]);
      elg_cart_coords(coord[0], coord[1], coord[2]);
    }
  else 
    {
      coord[0] = 1;
      coord[1] = 1;
      coord[2] = 1; 
      printf("P:%d My coordinates are %d %d %d\n", rank, coord[0], coord[1], coord[2]);
      elg_cart_coords(coord[0], coord[1], coord[2]);
    } 
}


void ompi_cart_topol(int rank, int size)
{
  int dim[3], period[3], coord[3];
  int tid, max_threads;
  
  /* run this program with 4 processes */
  if ( size != 4 ) 
    {
      fprintf(stderr, "Run this program with 4 processes\n");
      exit(0);
    }

  #if ( defined(ELG_OMPI) ) 
  max_threads = omp_get_max_threads();
  #endif  

  dim[0] = 2;
  dim[1] = 2;
  dim[2] = max_threads;
  
  period[0] = 1;
  period[1] = 0;
  period[2] = 1;
  

  /* set the user topology using elg_topol.c routine */
  elg_cart_create(dim[0], dim[1], dim[2] , period[0], period[1], period[2]);
    

  if ( rank == 0 ) 
    {
      coord[0] = 0;
      coord[1] = 0;     
    }
  else if ( rank == 1 ) 
    { 
      coord[0] = 0;
      coord[1] = 1;     	
    }
  else if ( rank == 2 )
    {
      coord[0] = 1;
      coord[1] = 0;    
    }
  else
    {
      coord[0] = 1;
      coord[1] = 1;      
    } 

   
#pragma omp parallel private(tid)
  {
#if ( defined(ELG_OMPI) )
    tid = omp_get_thread_num();  
#endif 
    coord[2] = tid;    
    printf("P:%d T:%d My coordinates are %d %d %d \n", 
	   rank, tid, coord[0], coord[1], coord[2]);     
    elg_cart_coords(coord[0], coord[1], tid);
  }
}


void omp_cart_topol()
{
  int dim[3], period[3], coord[3];
  int tid, max_threads;

  #if ( defined(ELG_OMP) )
  max_threads = omp_get_max_threads();  
  printf("max_threads = %d\n", max_threads);
  #endif  

  /* create a 1 dimensional cartesian topology */
  dim[0] = max_threads;  dim[1] = 0;  dim[2] = 0;  

  period[0] = 0; period[1] = 0 ; period[2] = 0;

  elg_cart_create(dim[0], dim[1], dim[2], period[0], period[1], period[2]);    


#pragma omp parallel private(tid)
  {  
    #if ( defined(ELG_OMP) )   
    tid = omp_get_thread_num();
    #endif

    coord[0] = tid; coord[1] = 0; coord[2] = 0;
    printf("T:%d My coordinates are %d %d %d \n", 
	   tid, coord[0], coord[1], coord[2]); 
    elg_cart_coords(coord[0], coord[1], coord[2]);
  }
}


int main(int argc, char **argv)
{ 
  int rank, size;
  int tid, max_threads, coord[3];

  #ifdef generic
  ELG_USER_START("main");
  #endif
  #ifdef pomp
  #pragma pomp inst init
  #pragma pomp inst begin(main)
  #endif
  
  #if (defined (ELG_MPI) || defined (ELG_OMPI))
  MPI_Init(&argc, &argv);
  MPI_Comm_rank(MPI_COMM_WORLD, &rank);
  MPI_Comm_size(MPI_COMM_WORLD, &size);
  #endif
  

  /* mpi_cart_topol is called for mpi_cart */  
  #if ( defined(ELG_MPI) )
  mpi_cart_topol(rank, size);
  #endif

  /* ompi_cart_topol is called for ompi_cart */
  #if ( defined(ELG_OMPI) )
  ompi_cart_topol(rank, size);
  #endif

  /* omp_cart_topol is called for omp_cart */
  #if ( defined(ELG_OMP) )
  omp_cart_topol();
  #endif

  #if (defined (ELG_MPI) || defined (ELG_OMPI))  
  MPI_Finalize();
  #endif

  #ifdef pomp
  #pragma pomp inst end(main)
  #endif
  #ifdef generic
  ELG_USER_END("main");
  #endif

  return 0; 
}

