/****************************************************************************
**  KOJAK                                                                  **
**  http://www.fz-juelich.de/zam/kojak/                                    **
*****************************************************************************
**  Copyright (c) 1998-2004                                                **
**  Forschungszentrum Juelich, Zentralinstitut fuer Angewandte Mathematik  **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "elg_trc.h"

extern void* vftr_getname();
extern int vftr_getname_len();

/*
 *-----------------------------------------------------------------------------
 *
 *  EPILOG Library (Event Processing, Investigating, and Logging)
 *
 *  - NEC SX ftrace compiler interface
 * 
 *-----------------------------------------------------------------------------
 */

/*
 *-----------------------------------------------------------------------------
 * Simple hash table to map function names to region identifier
 *-----------------------------------------------------------------------------
 */

typedef struct HN {
  long id;            /* hash code (address of function name) */
  elg_ui4 elgid;      /* associated EPILOG region identifier  */
  struct HN* next;
} HashNode;

#define HASH_MAX 1021

static int necsx_init = 1;       /* is initialization needed? */

static HashNode* htab[HASH_MAX];

/*
 * Stores EPILOG region identifier `e' under hash code `h'
 */

static void hash_put(long h, elg_ui4 e) {
  long id = h % HASH_MAX;
  HashNode *add = malloc(sizeof(HashNode));
  add->id = h;
  add->elgid = e;
  add->next = htab[id];
  htab[id] = add;
}

/*
 * Lookup hash code `h'
 * Returns EPILOG region identifier if already stored, otherwise ELG_NO_ID
 */

static elg_ui4 hash_get(long h) {
  long id = h % HASH_MAX;
  HashNode *curr = htab[id];
  while ( curr ) {
    if ( curr->id == h ) {
      return curr->elgid;
    }
    curr = curr->next;
  }
  return ELG_NO_ID;
}

/*
 * Register new region with EPILOG system
 */

static elg_ui4 register_region(char *func, int len) {
  elg_ui4 rid;
  static char fname[1024];

  strncpy(fname, func, len);
  fname[len] = '\0';
  rid = elg_def_region(fname, ELG_NO_ID, ELG_NO_LNO, ELG_NO_LNO,
                       "USR", ELG_FUNCTION);
  hash_put((long) func, rid);
  return rid;
}

/*
 * This function is called at the entry of each function
 * The call is generated by the NEC SX compilers
 */

void _ftrace_enter2_(void) {
  char *func = (char *)vftr_getname();
  int len = vftr_getname_len();
  elg_ui4 rid;

  /* -- if not yet initialized, initialize EPILOG -- */
  if ( necsx_init ) {
    necsx_init = 0;
    elg_open();
  }

  /* -- ignore NEC OMP runtime functions -- */
  if ( strchr(func, '$') != NULL ) return;

  /* -- get region identifier -- */
  if ( (rid = hash_get((long) func)) == ELG_NO_ID ) {
    /* -- region entered the first time, register region -- */
#   if defined (ELG_OMPI) || defined (ELG_OMP)
    if (omp_in_parallel()) {
#     pragma omp critical (elg_comp_ftrace_1)
      {
        if ( (rid = hash_get((long) func)) == ELG_NO_ID ) {
          rid = register_region(func, len);
        }
      }
    } else {
      rid = register_region(func, len);
    }
#   else
    rid = register_region(func, len);
#   endif
  }

  /* -- write EPILOG enter record -- */
  elg_enter(rid);
}

/*
 * This function is called at the exit of each function
 * The call is generated by the NEC SX compilers
 */

void _ftrace_exit2_(void) {
  char *func = (char *)vftr_getname();

  /* -- ignore NEC OMP runtime functions -- */
  if ( strchr(func, '$') != NULL ) return;

  /* -- write EPILOG exit record -- */
  elg_exit();
}

/*
 * This function is called at the exit of the program
 * The call is generated by the NEC SX compilers
 */

void _ftrace_stop2_(void) {
  /* -- write EPILOG exit record for main -- */
  elg_exit();
  elg_close();
}

