/****************************************************************************
**  KOJAK                                                                  **
**  http://www.fz-juelich.de/zam/kojak/                                    **
*****************************************************************************
**  Copyright (c) 1998-2004                                                **
**  Forschungszentrum Juelich, Zentralinstitut fuer Angewandte Mathematik  **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#include "elg_env.h"
#include "elg_error.h"
#include "elg_pform.h"

#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include <ctype.h>

static char* replace_vars(char *v) {
  char* start;
  char* end;
  char* vname;
  char* vval;
  char* res;
  int extra = 0;
  int plen = 0;

  if ((start = strchr(v, '$')) == NULL ) {
    /* no $ in v -> no replacement necessary */
    return v;
  } else {
    if ( start[1] == '{' ) {
      /* ${....} form */
      extra = 1;
      end = start += 2;
      while ( *end && *end != '}' ) end++;
    } else {
      /* $### form where # is letter, digit, or underscore */
      end = ++start;
      while ( *end && (isalnum(*end) || *end == '_')) end++;
    }
    /* determine name of variable */
    vname = malloc(end-start);
    strncpy(vname, start, end-start);
    vname[end-start] = '\0';

    /* get its content */
    if ((vval = getenv(vname)) == NULL) vval = "";

    /* put together string with variable replaced by value */
    /* -- allocate enough space and copy stuff before variable part */
    res = malloc(strlen(v)+strlen(vval));
    plen = (start - v) - 1 - extra;
    if (plen) strncpy(res, v, plen);
    res[plen] = '\0';
    /* -- add variable content */
    strcat(res, vval);
    /* -- add stuff after variable */
    if ( *end ) strcat(res, end + extra);
    return res;
  }
}

char* elg_env_gdir()
{
  static char* gdir = NULL;
  char* tmp;

  if (! gdir)
    {
      tmp = getenv("ELG_PFORM_GDIR");
      if (tmp != NULL)
        {
          gdir = replace_vars(tmp);
        }
      else
        {
          gdir = replace_vars(elg_pform_gdir());
        } 
    }
  return gdir;
}
char* elg_env_ldir()
{
  static char* ldir = NULL;
  char* tmp;

  if (! ldir)
    {
      tmp = getenv("ELG_PFORM_LDIR");
      if (tmp != NULL)
        {
          ldir = replace_vars(tmp);
        }
      else
        {
          ldir = replace_vars(elg_pform_ldir());
        } 
    }
  return ldir;
}

char* elg_env_fprefix()
{
  static char* fprefix = NULL;
  char* tmp;

  if (! fprefix)
    {
      tmp = getenv("ELG_FILE_PREFIX");
      if (tmp != NULL)
        {
          fprefix = replace_vars(tmp);
        }
      else
        {
          fprefix = "a";
        } 
    }
  return fprefix;
}

size_t elg_env_bsize()
{
  size_t buffer_size;

  char* tmp = getenv("ELG_BUFFER_SIZE");
  if (tmp != NULL)
    {
      buffer_size = atoi(tmp);
      if (buffer_size <= 0)
	  elg_error_msg("ELG_BUFFER_SIZE not properly set");
      else
	  return buffer_size;
    }
  return 10000000;
}

int elg_env_is_verbose()
{
  char* tmp = getenv("ELG_VERBOSE");
  if ((tmp != NULL) &&
      strcmp(tmp, "NO") != 0 &&
      strcmp(tmp, "no") != 0 && 
      strcmp(tmp, "No") != 0 && 
      strcmp(tmp, "0") != 0)
    {
      return 1;
    }
  else
    {
      return 0;
    }
}

int elg_env_do_merge()
{
  char* tmp = getenv("ELG_MERGE");
  if ((tmp != NULL) &&
      (strcmp(tmp, "NO") == 0 ||
       strcmp(tmp, "no") == 0 || 
       strcmp(tmp, "No") == 0 || 
       strcmp(tmp, "0") == 0))
    {
      return 0;
    }
  else
    {
      return 1;
    }
}

char* elg_env_metrics()
{
  return getenv("ELG_METRICS");
}

long elg_env_max_reg_call()
{
  int maxcall;

  char* tmp = getenv("ELG_MAX_REGION_CALLCOUNT");
  if (tmp != NULL)
    {
      if ( strcmp(tmp, "unlimited") == 0 ||
           strcmp(tmp, "Unlimited") == 0 ||
           strcmp(tmp, "UNLIMITED") == 0 )
        {
	  return LONG_MAX;
        }
      else
        {
           maxcall = atoi(tmp);
           if (maxcall <= 0)
	       elg_error_msg("ELG_MAX_REGION_CALLCOUNT not properly set");
           else
	       return maxcall;
	}
    }
  return 5000L;
}

char* elg_env_blist()
{
  return getenv("ELG_BLACKLIST");
}
