/****************************************************************************
**  KOJAK                                                                  **
**  http://www.fz-juelich.de/zam/kojak/                                    **
*****************************************************************************
**  Copyright (c) 1998-2004                                                **
**  Forschungszentrum Juelich, Zentralinstitut fuer Angewandte Mathematik  **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#ifndef _ELG_MPIREQ_H
#define _ELG_MPIREQ_H

#ifdef __cplusplus
#   define EXTERN extern "C" 
#else
#   define EXTERN extern 
#endif

#include "elg_defs.h"
#include "mpi.h"

/* 
 *-----------------------------------------------------------------------------
 *
 *  EPILOG Library (Event Processing, Investigating, and Logging)
 *
 *  - MPI request management
 *
 *-----------------------------------------------------------------------------
 */

enum ElgReqFlags {
  ERF_NONE = 0x00,
  ERF_SEND = 0x01,
  ERF_RECV = 0x02,
  ERF_IS_PERSISTENT = 0x10,
  ERF_DEALLOCATE = 0x20,
  ERF_IS_ACTIVE  = 0x40
};

struct ElgRequest {
  MPI_Request request;
  unsigned flags;
  int dest;
  int bytes;
  MPI_Comm comm;
  int tag;
};

EXTERN void elg_request_finalize();
EXTERN void elg_request_create( MPI_Request request, 
				unsigned flags,
				int tag, 
				int dest, 
				int bytes, 
				MPI_Comm comm );
EXTERN struct ElgRequest* elg_request_get(MPI_Request request);
EXTERN void elg_request_free(struct ElgRequest* req);
EXTERN void elg_check_request(struct ElgRequest* req, MPI_Status *status);
EXTERN void elg_save_request_array(MPI_Request *arr_req, int arr_req_size);
EXTERN struct ElgRequest* elg_saved_request_get(int i);

#endif
