/****************************************************************************
**  KOJAK                                                                  **
**  http://www.fz-juelich.de/zam/kojak/                                    **
*****************************************************************************
**  Copyright (c) 1998-2004                                                **
**  Forschungszentrum Juelich, Zentralinstitut fuer Angewandte Mathematik  **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#include <stdio.h>

#include "elg_pform.h"
#include "elg_defs.h"
#include <bglpersonality.h>
#include <rts.h>

static double elg_clockspeed=1.0e-6/700.0;

static BGLPersonality mybgl;

/* platform specific initialization */
void elg_pform_init() {
  rts_get_personality(&mybgl, sizeof(BGLPersonality));
}

void elg_pform_mpi_init() {}

void elg_pform_mpi_finalize() {} 

/* directory of global file system  */
char* elg_pform_gdir() {
  return ".";
}

/* directory of local file system  */
char* elg_pform_ldir() {
  #ifdef PFORM_LDIR
    return PFORM_LDIR;
  #else
    return ".";
  #endif
}

/* is a global clock provided ? */ 
int elg_pform_is_gclock() {
  return 1;
}

/* local or global wall-clock time in seconds */
double elg_pform_wtime() {
  return ( rts_get_timebase() * elg_clockspeed );
}

/* platform name */
char*  elg_pform_name() {
  return "IBM BG/L";
}

/* number of nodes */
int elg_pform_num_nodes() {
  if (mybgl.virtualNodeMode)
    return mybgl.numPsets * mybgl.numNodesInPset * mybgl.virtualNodeMode * 2;
  else
    return mybgl.numPsets * mybgl.numNodesInPset * mybgl.virtualNodeMode;
}

/* unique numeric SMP-node identifier */
long elg_pform_node_id() {
  if (mybgl.virtualNodeMode)
    return (mybgl.psetNum * mybgl.numNodesInPset + mybgl.rankInPset)*2
           + rts_get_processor_id();
  else
    return mybgl.psetNum * mybgl.numNodesInPset + mybgl.rankInPset;
}

static void bgl_getLocString(const BGLPersonality *p, char *buf) {
  unsigned row = (p->location >> 15) & 0xf;
  unsigned col = (p->location >> 11) & 0xf;
  unsigned mp  = (p->location >> 10) & 1;
  unsigned nc  = (p->location >> 6) & 0xf;
  unsigned pc  = (p->location >> 1) & 0x1f;
  unsigned asic = p->location & 1;
  const char *asicname = (asic ? "U01" : "U11");
  if (row == 0xff)
    sprintf(buf, "Rxx-Mx-N%x-J%02d-%s", nc, pc, asicname);
  else
    sprintf(buf, "R%x%x-M%d-N%x-J%02d-%s", row, col, mp, nc, pc, asicname);
}

/* unique string SMP-node identifier */
char* elg_pform_node_name() {
  static char node[128];
  sprintf(node, "node-%03d-%03d-%03d-%d", mybgl.xCoord, mybgl.yCoord,
          mybgl.zCoord, rts_get_processor_id());
/*
  bgl_getLocString(&mybgl, node);
*/
  return node;              
}

/* number of CPUs */
int elg_pform_num_cpus() {
  return 1;
}
