/****************************************************************************
**  KOJAK                                                                  **
**  http://www.fz-juelich.de/zam/kojak/                                    **
*****************************************************************************
**  Copyright (C) 1998-2003                                                **
**  Forschungszentrum Juelich, Zentralinstitut fuer Angewandte Mathematik  **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "elg_pomp2_grp.h"

/*
 * Determine POMP Group
 */
enum pomp_group_t pomp_group(char *str) {
  switch  ( str[0] ) {
  case 'L': if ( strcmp(str, "LOOP") == 0 ) return GRP_LOOP;
	    break;
  case 'P': if ( strcmp(str, "PARALLEL") == 0 ) return GRP_PARALLEL;
	    break;
  case 'R': if ( strcmp(str, "RUNTIME") == 0 ) return GRP_RUNTIME;
	    break;
  case 'S': if ( strcmp(str, "SYNC") == 0 ) return GRP_SYNC;
	    break;
  case 'U': if ( strcmp(str, "USER") == 0 ) return GRP_USER;
	    break;
  case 'W': if ( strcmp(str, "WORKSHARE") == 0 ) return GRP_WSHARE;
	    break;
  }
  return GRP_UNKNOWN;
}

/*
 * Determine POMP Group Value
 */
enum pomp_grpval_t pomp_grpval(char *str) {
  switch  ( str[0] ) {
  case 'b':
  case 'B': if ( strcasecmp(str, "BeginEnd") == 0 ) return GV_BEGINEND;
	    break;
  case 'e':
  case 'E': if ( strcasecmp(str, "Event") == 0 ) return GV_EVENT;
            if ( strcasecmp(str, "EnterExit") == 0 ) return GV_ENTEREXIT;
	    break;
  case 'i':
  case 'I': if ( strcasecmp(str, "IterBeginEnd") == 0 ) return GV_ITER_BEGINEND;
            if ( strcasecmp(str, "IterChunk") == 0 ) return GV_ITER_CHUNK;
            if ( strcasecmp(str, "IterEvent") == 0 ) return GV_ITER_EVENT;
	    break;
  case 'n':
  case 'N': if ( strcasecmp(str, "None") == 0 ) return GV_NONE;
	    break;
  }
  return GV_UNKNOWN;
}


/*
 * Determine POMP Group Environment
 */
enum pomp_grpval_t pomp_grp_level[] = {
  /* GRP_PARALLEL */ GV_BEGINEND,
  /* GRP_LOOP     */ GV_ENTEREXIT,
  /* GRP_WSHARE   */ GV_BEGINEND,
  /* GRP_SYNC     */ GV_BEGINEND,
  /* GRP_USER     */ GV_BEGINEND,
  /* GRP_RUNTIME  */ GV_BEGINEND
};

static enum pomp_grpval_t check_envvar(char *evar, enum pomp_grpval_t deflt) {
  char *val;
  enum pomp_grpval_t gval;

  if ( ((val = getenv(evar)) != NULL) &&
       ((gval = pomp_grpval(val)) != GV_UNKNOWN) ) {
    fprintf(stderr, "+++: %s=%d\n", evar, gval);
    return gval;
  }
  return deflt;
}

void pomp_init_groups() {
  pomp_grp_level[GRP_PARALLEL] = check_envvar("POMP_PARALLEL", GV_BEGINEND);
  pomp_grp_level[GRP_LOOP]     = check_envvar("POMP_LOOP", GV_ENTEREXIT);
  pomp_grp_level[GRP_WSHARE]   = check_envvar("POMP_WORKSHARE", GV_BEGINEND);
  pomp_grp_level[GRP_SYNC]     = check_envvar("POMP_SYNC", GV_BEGINEND);
  pomp_grp_level[GRP_USER]     = check_envvar("POMP_USER", GV_BEGINEND);
  pomp_grp_level[GRP_RUNTIME]  = check_envvar("POMP_RUNTIME", GV_EVENT);
}
