/****************************************************************************
**  KOJAK                                                                  **
**  http://www.fz-juelich.de/zam/kojak/                                    **
*****************************************************************************
**  Copyright (c) 1998-2004                                                **
**  Forschungszentrum Juelich, Zentralinstitut fuer Angewandte Mathematik  **
**                                                                         **
**  Copyright (c) 2003-2004                                                **
**  University of Tennessee                                                **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#include <stdlib.h>
#include "elg_error.h"
#include "elg_readcb.h"

void elg_readcb_ALL( elg_ui1 type, 
		   elg_ui1 length,
                   void* userdata )
{}

void elg_readcb_STRING( elg_ui4 strid,      /* string identifier */
		      elg_ui1 cntc,         /* number of continuation records */
		      const char* str,      /* string */
                      void* userdata ) 
{}

void elg_readcb_STRING_CNT( const char* str, /* continued string */
                          void* userdata ) 
{}

void elg_readcb_MACHINE( elg_ui4 mid,       /* machine identifier */
		       elg_ui4 nodec,       /* number of nodes */
		       elg_ui4 mnid,        /* machine name identifier */
                       void* userdata ) 
{}

void elg_readcb_NODE( elg_ui4 nid,          /* node identifier */
		    elg_ui4 mid,            /* machine identifier */
		    elg_ui4 cpuc,           /* number of CPUs */
		    elg_ui4 nnid,           /* node name identifer */
		    elg_d8  cr,             /* number of clock cycles/sec */
                    void* userdata ) 
{}

void elg_readcb_PROCESS( elg_ui4 pid,       /* process identifier */
		       elg_ui4 pnid,        /* process name identifier */
                       void* userdata ) 
{}

void elg_readcb_THREAD( elg_ui4 tid,        /* thread identifier */
		      elg_ui4 pid,          /* process identifier */
		      elg_ui4 tnid,         /* thread name identifier */
                      void* userdata ) 
{}

void elg_readcb_LOCATION( elg_ui4 lid,      /* location identifier */
			elg_ui4 mid,        /* machine identifier */
			elg_ui4 nid,        /* node identifier */
			elg_ui4 pid,        /* process identifier */
			elg_ui4 tid,        /* thread identifier */
                        void* userdata ) 
{}

void elg_readcb_FILE( elg_ui4 fid,          /* file identifier */
		    elg_ui4 fnid,           /* file name identifier */
                    void* userdata ) 
{}

void elg_readcb_REGION( elg_ui4 rid,        /* region identifier */
		      elg_ui4 rnid,         /* region name identifier */
		      elg_ui4 fid,          /* source file identifier */
		      elg_ui4 begln,        /* begin line number */
		      elg_ui4 endln,        /* end line number */
		      elg_ui4 rdid,         /* region description identifier */
		      elg_ui1 rtype,        /* region type */
                      void* userdata ) 
{}

void elg_readcb_CALL_SITE( elg_ui4 csid,    /* call site identifier */
			 elg_ui4 fid,       /* source file identifier */
			 elg_ui4 lno,       /* line number */
			 elg_ui4 erid,      /* region identifer to be entered */
			 elg_ui4 lrid,      /* region identifer to be left */
                         void* userdata ) 
{}

void elg_readcb_METRIC( elg_ui4 metid,      /* metric identifier */
		      elg_ui4 metnid,       /* metric name identifier */
		      elg_ui4 metdid,       /* metric description identifier */
		      elg_ui1 metdtype,     /* metric data type */
		      elg_ui1 metmode,      /* metric mode */
		      elg_ui1 metiv,        /* time interval referenced */
                      void* userdata ) 
{}

void elg_readcb_MPI_COMM( elg_ui4 cid,      /* communicator identifier */
			elg_ui4 grpc,       /* size if bitstring in bytes */
			elg_ui1 grpv[],     /* bitstring defining the group */
                        void* userdata ) 
{}

void elg_readcb_CART_TOPOLOGY(elg_ui4 topid,   /* topology id */ 
			    elg_ui4 ndims,     /* number of dimensions */
			    elg_ui4 dimv[],    /* number of processes in each dim */  
			    elg_ui1 periodv[], /* periodicity in each dim */
			    void* userdata)      
{}

void elg_readcb_CART_COORDS   ( elg_ui4 topid,  /* topology id */
			      elg_ui4 lid,      /* location id */
			      elg_ui1 ndims,    /* number of dimensions */
			      elg_ui4 coordv[], /* coordinates in each dimension */
			      void* userdata)
{}

void elg_readcb_OFFSET( elg_d8  ltime,      /* local time */
		      elg_d8  offset,       /* offset to global time */
                      void* userdata ) 
{}

void elg_readcb_NUM_EVENTS( elg_ui4 eventc, /* number of event records */
                          void* userdata ) 
{}

void elg_readcb_LAST_DEF( void* userdata ) 
{}


void elg_readcb_ENTER( elg_ui4 lid,         /* location identifier */
		     elg_d8  time,          /* time stamp */
		     elg_ui4 rid,           /* region identifier entered */
		     elg_ui1 metc,          /* number of metrics */
		     elg_ui8 metv[],        /* metric values */
                     void* userdata ) 
{}

void elg_readcb_ENTER_CS( elg_ui4 lid,      /* location identifier */
			elg_d8  time,       /* time stamp */
			elg_ui4 csid,       /* call site identifier called */
			elg_ui1 metc,       /* number of metrics */
			elg_ui8 metv[],     /* metric values */
                        void* userdata ) 
{}

void elg_readcb_EXIT( elg_ui4 lid,          /* location identifier */
		    elg_d8  time,           /* time stamp */
		    elg_ui1 metc,           /* number of metrics */
		    elg_ui8 metv[],         /* metric values */
                    void* userdata ) 
{}

void elg_readcb_MPI_SEND( elg_ui4 lid,      /* location identifier */
			elg_d8  time,       /* time stamp */
			elg_ui4 dlid,       /* destination location identifer */
			elg_ui4 cid,        /* communicator identifier */
			elg_ui4 tag,        /* message tag */
			elg_ui4 sent,       /* message length in bythes */
                        void* userdata ) 
{}

void elg_readcb_MPI_RECV( elg_ui4 lid,      /* location identifier */
			elg_d8  time,       /* time stamp */
			elg_ui4 slid,       /* source location identifier */
			elg_ui4 cid,        /* communicator identifier */
			elg_ui4 tag,        /* message tag */
                        void* userdata ) 
{}

void elg_readcb_MPI_COLLEXIT( elg_ui4 lid,  /* location identifier */
			    elg_d8  time,   /* time stamp */
			    elg_ui1 metc,   /* number of metrics */
			    elg_ui8 metv[], /* metric values */
			    elg_ui4 rlid,   /* root location identifier */
			    elg_ui4 cid,    /* communicator identifier */
			    elg_ui4 sent,   /* bytes sent */
			    elg_ui4 recvd,  /* bytes received */
                            void* userdata ) 
{}

void elg_readcb_OMP_FORK( elg_ui4 lid,      /* location identifier */
			elg_d8  time,       /* time stamp */
                        void* userdata ) 
{}

void elg_readcb_OMP_JOIN( elg_ui4 lid,      /* location identifier */
			elg_d8  time,       /* time stamp */
                        void* userdata ) 
{}

void elg_readcb_OMP_ALOCK( elg_ui4 lid,     /* location identifier */
			 elg_d8  time,      /* time stamp */
			 elg_ui4 lkid,      /* lock identifier acquired */
                         void* userdata ) 
{}

void elg_readcb_OMP_RLOCK( elg_ui4 lid,     /* location identifier */
			 elg_d8  time,      /* time stamp */
			 elg_ui4 lkid,      /* lock identifier released */
                         void* userdata ) 
{}

void elg_readcb_OMP_COLLEXIT( elg_ui4 lid,  /* location identifier */
			    elg_d8  time,   /* time stamp */
			    elg_ui1 metc,   /* number of metrics */
			    elg_ui8 metv[], /* metric values */
                            void* userdata ) 
{}

void elg_readcb_ENTER_DUMP( elg_ui4 lid,    /* location identifier */
			  elg_d8  time,     /* time stamp */
			  elg_ui1 metc,     /* number of metrics */
			  elg_ui8 metv[],   /* metric values */
                          void* userdata ) 
{}

void elg_readcb_EXIT_DUMP( elg_ui4 lid,     /* location identifier */
			 elg_d8  time,      /* time stamp */
			 elg_ui1 metc,      /* number of metrics */
			 elg_ui8 metv[],    /* metric values */
                         void* userdata ) 
{}

void elg_readcb_LOG_OFF( elg_ui4 lid,       /* location identifier */
		       elg_d8  time,        /* time stamp */
		       elg_ui1 metc,        /* number of metrics */
		       elg_ui8 metv[],      /* metric values */
                       void* userdata ) 
{}

void elg_readcb_LOG_ON( elg_ui4 lid,        /* location identifier */
		      elg_d8  time,         /* time stamp */
		      elg_ui1 metc,         /* number of metrics */
		      elg_ui8 metv[],       /* metric values */
                      void* userdata ) 
{}

/*
 * Example function readiung an EPILOG trace file
 */
void read_epilog_trace(char *file) 
{
  ElgRCB* handle;

  /* Open EPILOG trace stored in `file' */
  handle = elg_read_open(file);
  if (handle == NULL) return;

  /* Process every event record of the trace */
  /* Calls read callback functions above     */
  while ( elg_read_next(handle, NULL) );

  /* Close trace file and cleanup */
  elg_read_close(handle);
}



















