from Severity import *

class Profile:

    def __init__(self, trace, analyzer):

        self._trace = trace

        self._profile = {}
        self._profile['time']    = Severity(self._trace.get_nlocs())
        self._profile['multhrd'] = Severity(self._trace.get_nlocs())

        self._met_namev = []
        for m in range(self._trace.get_nmets()):
            self._met_namev.append(self._trace.get_met(m)['name'])
            self._profile[self._met_namev[m]] = Severity(self._trace.get_nlocs())

        self._ismulthrd = []
        for i in range(self._trace.get_nprocs()):
            self._ismulthrd.append(0)

        analyzer.subscribe('FORK', self.fork)
        analyzer.subscribe('JOIN', self.join)
        analyzer.subscribe('EXIT', self.exit)
        
    def fork(self, fork):
        
        loc_id  = fork['loc_id']
        proc_id  = self._trace.get_loc(loc_id)['proc_id']
        self._ismulthrd[proc_id] = 1

    def join(self, join):
        
        loc_id  = join['loc_id']
        proc_id  = self._trace.get_loc(loc_id)['proc_id']
        self._ismulthrd[proc_id] = 0

    def exit(self, exit):
        
        loc_id   = exit['loc_id']
        proc_id  = self._trace.get_loc(loc_id)['proc_id']
        pos      = exit['pos']
        enter    = self._trace.event(exit['enterptr']) 
        cnode_id = enter['cnodeptr']
        
        duration = exit['time'] - enter['time']
        self._profile['time'].add(cnode_id, loc_id, duration)
        for m in range(self._trace.get_nmets()):
            met_name = self._met_namev[m]
            difference  = exit[met_name] - enter[met_name] 
            self._profile[met_name].add(cnode_id, loc_id, difference)

        # identify multithreaded tuples (call path, location)
        if self._ismulthrd[proc_id] == 0:
            # !! no EXIT event for slave threads outside parallel regions
            for thrd_id in range(self._trace.get_proc(proc_id)['nthrds']):
                loc_id = self._trace.get_thrd(proc_id, thrd_id)['loc_id']
                self._profile['multhrd'].set(cnode_id, loc_id, 0)
        else:
            self._profile['multhrd'].set(cnode_id, loc_id, 1)

    def exclusify(self, ctree):
        # exclusify
        self._profile['time'].exclusify( ctree)
        for m in range(self._trace.get_nmets()):
            self._profile[self._met_namev[m]].exclusify(ctree)
            
        # outside parallel regions set profile of slave threads to zero
        for cnode_id in ctree.keys():
            for loc_id in range(self._trace.get_nlocs()):
                # if slave thread outside a parallel region 
                if ( not self.is_multhrd(cnode_id, loc_id) ) and self._trace.get_loc(loc_id)['thrd_id'] != 0:
                    self._profile['time'].set(cnode_id, loc_id, 0)
                    for m in range(self._trace.get_nmets()):
                        self._profile[self._met_namev[m]].set(cnode_id, loc_id, 0)
            
        
    def get_time(self, cnode_id = None, loc_id = None):
        return  self._profile['time'].get(cnode_id, loc_id)

    def get_metr(self, met_name, cnode_id = None, loc_id = None):
        return  self._profile[met_name].get(cnode_id, loc_id)

    def is_multhrd(self, cnode_id, loc_id):
        return  self._profile['multhrd'].get(cnode_id, loc_id)

