/****************************************************************************
**  KOJAK                                                                  **
**  http://www.fz-juelich.de/zam/kojak/                                    **
*****************************************************************************
**  Copyright (C) 1998-2004                                                **
**  Forschungszentrum Juelich, Zentralinstitut fuer Angewandte Mathematik  **
**                                                                         **
**  Copyright (c) 2003-2004                                                **
**  University of Tennessee, Innovative Computing Laboratory               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#ifndef EXP_COMPOUNDEVENTS_H
#define EXP_COMPOUNDEVENTS_H

enum cetype { RECV_OP, 
	      LATE_SEND,
	      LATE_RECV,
	      MPI_COLL,
	      TEAM_CREATE,
	      LOCK_COMP,
	      OMP_COLL };

struct cmp_event 
{
  virtual ~cmp_event() { }
  cetype type;
};

/****************************************************************************
* Receive operation
****************************************************************************/

struct recv_op : public cmp_event
{
  recv_op() { type = RECV_OP; }

  earl::Event recv;           // RECV event
  earl::Event enter_recv;     // ENTER event of receive operation
  const earl::Region* region; // receive operation 
};

/****************************************************************************
* Late sender
****************************************************************************/

struct late_send : public recv_op
{
  late_send() { type = LATE_SEND; }
  late_send(const recv_op* recv_op) 
  {
    type = LATE_SEND;
    recv = recv_op->recv;  
    enter_recv = recv_op->enter_recv;    
    region = recv_op->region;   
  }
  
  earl::Event send;           // SEND event
  earl::Event enter_send;     // ENTER event of send operation      
  double      idle_time;      // time receiver waited for message
};

/****************************************************************************
* Late receiver
****************************************************************************/

struct late_recv : public recv_op
{
  late_recv() { type = LATE_RECV; }
  late_recv(const recv_op* recv_op) 
  {
    type = LATE_RECV; 
    recv = recv_op->recv;  
    enter_recv = recv_op->enter_recv;    
    region = recv_op->region;   
  }

  earl::Event send;           // SEND event
  earl::Event enter_send;     // ENTER event of send operation      
  double      idle_time;      // time sender waited for receiver
};

/****************************************************************************
* MPI collective operation
****************************************************************************/

struct mpi_coll : public cmp_event
{
  mpi_coll() { type = MPI_COLL; }

  std::vector<earl::Event> exitv;     // list of EXIT events ( in ascending rank order )
  std::vector<earl::Event> enterv;    // list of ENTER events ( in ascending rank order )
  long                     root_idx;  // list index of root location
  earl::Region*            region;    // collective operation               
};

/****************************************************************************
* Team creation
****************************************************************************/

struct team_create : public cmp_event
{
  team_create() { type = TEAM_CREATE; }

  earl::Event enter;         // ENTER event of parallel region
  double      time;          // time difference between FORK and ENTER event 
};

/****************************************************************************
* Lock competion
****************************************************************************/

struct lock_comp : public cmp_event
{
  lock_comp() { type = LOCK_COMP; }

  earl::Event alock;         // ALOCK event
  earl::Event enter_alock;   // ENTER event of aquire-lock operation
  earl::Event rlock;         // RLOCK event
  double      idle_time;     // time thread has to wait for lock acquisition
};

/****************************************************************************
* OpenMP collective operation
****************************************************************************/

struct omp_coll : public cmp_event
{
  omp_coll() { type = OMP_COLL; }

  std::vector<earl::Event> exitv;     // list of EXIT events ( in ascending rank order )
  std::vector<earl::Event> enterv;    // list of ENTER events ( in ascending rank order )
  earl::Region*            region;    // collective operation               
};

#endif
